#!#%PERL.PATH%#

###############################################################################
# Panel.cgi                                                                   #
###############################################################################
# UltraBoard 2000 by UltraScripts.com, Inc.                                   #
# Scripts written by Jacky Wing Wa Yung, WebMaster@UltraScripts.com           #
# Available from http://www.UltraBoard.org/                                   #
# =========================================================================== #
# COPYRIGHT NOTICE :                                                          #
#                                                                             #
# Copyright (c) 1999-2000 UltraScripts.com, Inc. All Rights Reserved.         #
#                                                                             #
# This program is being distributed as shareware. It may be used and modified #
# free for personal, government or non-profit. Any commercial use must be     #
# registered. However, modified versions cannot be sold or distributed. You   #
# cannot alter the copyright and "powered by" notices throughout the scripts. #
# These notices must be clearly visible to the end users.                     #
#                                                                             #
# WARRANTY DISCLAIMER:                                                        #
#                                                                             #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT #
# ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR       #
# FITNESS FOR A PARTICULAR PURPOSE.                                           #
###############################################################################

package UB;

use CGI qw(:cgi);
use strict;

BEGIN {
  # if you having problem on the script, set $UB::PATH to the full path to your
  # control panel directory.
  $UB::PATH    = "#%UB.PATH%#";

  $SYSTEM::EXT = "#%PERL.EXT%#" || $ENV{'SCRIPT_NAME'} || $ENV{'SCRIPT_FILENAME'} || $0;
  $SYSTEM::EXT =~s/^.+\.(\w+)$/$1/g;

  ($0 =~ m/(.*)\/[^\/]+/) && unshift (@INC, "$1");
  ($0 =~ m/(.*)\\[^\\]+/) && unshift (@INC, "$1");

  eval {
    unless (-e $UB::PATH."/Panel.$SYSTEM::EXT") {
      foreach my $P ( $ENV{'SCRIPT_FILENAME'}, $ENV{'PATH_TRANSLATED'}, $ENV{'DOCUMENT_ROOT'}.$ENV{'SCRIPT_NAME'}, $0 ) {
        $P =~s/\\/\//g;
        $P =~s/(.*)\/[^\/]+/$1/g;
        if (-e $P."/Panel.$SYSTEM::EXT") {
          $UB::PATH = $P;
          last;
        }
      }
    }
  };

  unshift (@INC, "$UB::PATH");
}

# required using at least perl v5.004.
eval {
  require 5.004;
} || &UB::CGIError("Can't load the perl v5.004 or higher version. $@", "");

###############################################################################
# Variables

$| = 1;

$UB::VERSION         = "2.00";

$UB::SOURCES_PATH    = "$UB::PATH/Private/Sources";
$UB::VARIABLES_PATH  = "$UB::PATH/Private/Variables";
$UB::LIBRARIES_PATH  = "$UB::SOURCES_PATH/Libraries";
$UB::INTERFACES_PATH = "$UB::PATH/Private/Interfaces";

$UB::CONTENT_TYPE    = 0;

%UB::MEMBER          = ();
%UB::COOKIES         = ();
%UB::QUERY           = ();

push (@INC, $UB::LIBRARIES_PATH);

###############################################################################
# Loading Required Libraries

unless (my $Return = do "$UB::VARIABLES_PATH/System.cfg") {
  &UB::CGIError("Couldn't parse system variables. $@", "$UB::VARIABLES_PATH/System.cfg") if ($@);
  &UB::CGIError("Couldn't do system variables. $_"   , "$UB::VARIABLES_PATH/System.cfg") unless (defined $Return);
  &UB::CGIError("Couldn't run system variables."     , "$UB::VARIABLES_PATH/System.cfg") unless ($Return);
}

unless (my $Return = do "$UB::VARIABLES_PATH/General.cfg") {
  &UB::CGIError("Couldn't parse general variables. $@", "$UB::VARIABLES_PATH/General.cfg") if ($@);
  &UB::CGIError("Couldn't do general variables. $_"   , "$UB::VARIABLES_PATH/General.cfg") unless (defined $Return);
  &UB::CGIError("Couldn't run general variables"      , "$UB::VARIABLES_PATH/General.cfg") unless ($Return);
}

unless (my $Return = do "$SYSTEM::THEME_PATH/$SYSTEM::DEFAULT_THEME/Style/Style.cfg") {
  &UB::CGIError("Couldn't parse style variables. $@", "$SYSTEM::THEME_PATH/$SYSTEM::DEFAULT_THEME/Style/Style.cfg") if ($@);
  &UB::CGIError("Couldn't do style variables. $_"   , "$SYSTEM::THEME_PATH/$SYSTEM::DEFAULT_THEME/Style/Style.cfg") unless (defined $Return);
  &UB::CGIError("Couldn't run style variables"      , "$SYSTEM::THEME_PATH/$SYSTEM::DEFAULT_THEME/Style/Style.cfg") unless ($Return);
}

# loading standard library.
eval { require "$UB::LIBRARIES_PATH/Standard.lib.pl"       } || &UB::CGIError("Can't load standard libraries for control panel. $@", "$UB::LIBRARIES_PATH/Standard.lib.pl"      );
# loading database library.
eval { require "$UB::LIBRARIES_PATH/Database.lib.pl"       } || &UB::CGIError("Can't load database libraries for control panel. $@", "$UB::LIBRARIES_PATH/Database.lib.pl"      );
# loading interfaces sources.
eval { require "$UB::INTERFACES_PATH/CP_Interfaces.src" } || &UB::CGIError("Can't load interfaces for control panel. $@"        , "$UB::INTERFACES_PATH/CP_Interfaces.src");

# error running main function
eval { &UB::Main } || &UB::CGIError("Can't run your request. $@", "");

###############################################################################
# Main function
# - function of executing all functions
sub Main {
  $UB::CGI = new CGI;

  %UB::QUERY = ();
  foreach my $Keyword ($UB::CGI->param()) {
    $UB::QUERY{ $Keyword } = $UB::CGI->param( $Keyword );
    $UB::QUERY{ $Keyword } =~ s/^\s+//g;
    $UB::QUERY{ $Keyword } =~ s/\s+$//g;
  }

  %UB::COOKIES = $UB::CGI->cookie('UB2000');

  my $action = $UB::QUERY{'action'};

  unless (&Standard::Authenticate()) {
    require "$UB::SOURCES_PATH/CP_Login.pl";
    &Source::Login();
    exit;
  }

  if    ((!defined $action) || ($action eq "")) {
    &HTML::ControlPanel();
  }
  elsif ($action eq "menu") {
    &HTML::CPMenu();
  }
  elsif ($action eq "body") {
    require "$UB::SOURCES_PATH/CP_Index.pl";
    &Source::Index();
  }
  elsif ($action eq "DoDisApprovePosts") {
    require "$UB::SOURCES_PATH/CP_DoDisApprovePosts.pl";
    &Source::DoDisApprovePosts();
  }
  elsif ($action eq "DoDisApproveAccounts") {
    require "$UB::SOURCES_PATH/CP_DoDisApproveAccounts.pl";
    &Source::DoDisApproveAccounts();
  }

  elsif ($action eq "CfgSystem") {
    if ($UB::MEMBER{'LEVEL'} < 100) {
      require "$UB::SOURCES_PATH/CP_Login.pl";
      &Source::Login();
      exit;
    }

    require "$UB::SOURCES_PATH/CP_CfgSystem.pl";
    &Source::CfgSystem();
  }
  elsif ($action eq "DoCfgSystem") {
    if ($UB::MEMBER{'LEVEL'} < 100) {
      require "$UB::SOURCES_PATH/CP_Login.pl";
      &Source::Login();
      exit;
    }

    require "$UB::SOURCES_PATH/CP_DoCfgSystem.pl";
    &Source::DoCfgSystem();
  }
  elsif ($action eq "CfgGeneral") {
    if ($UB::MEMBER{'LEVEL'} < 100) {
      require "$UB::SOURCES_PATH/CP_Login.pl";
      &Source::Login();
      exit;
    }

    require "$UB::SOURCES_PATH/CP_CfgGeneral.pl";
    &Source::CfgGeneral();
  }
  elsif ($action eq "DoCfgGeneral") {
    if ($UB::MEMBER{'LEVEL'} < 100) {
      require "$UB::SOURCES_PATH/CP_Login.pl";
      &Source::Login();
      exit;
    }

    require "$UB::SOURCES_PATH/CP_DoCfgGeneral.pl";
    &Source::DoCfgGeneral();
  }
  elsif ($action eq "CfgStyle") {
    if ($UB::MEMBER{'LEVEL'} < 100) {
      require "$UB::SOURCES_PATH/CP_Login.pl";
      &Source::Login();
      exit;
    }

    require "$UB::SOURCES_PATH/CP_CfgStyle.pl";
    &Source::CfgStyle();
  }
  elsif ($action eq "DoCfgStyle") {
    if ($UB::MEMBER{'LEVEL'} < 100) {
      require "$UB::SOURCES_PATH/CP_Login.pl";
      &Source::Login();
      exit;
    }

    require "$UB::SOURCES_PATH/CP_DoCfgStyle.pl";
    &Source::DoCfgStyle();
  }
  elsif ($action eq "ModifyAdministrator") {
    if ($UB::MEMBER{'LEVEL'} < 100) {
      require "$UB::SOURCES_PATH/CP_Login.pl";
      &Source::Login();
      exit;
    }

    require "$UB::SOURCES_PATH/CP_ModifyAdministrator.pl";
    &Source::ModifyAdministrator();
  }
  elsif ($action eq "DoModifyAdministrator") {
    require "$UB::SOURCES_PATH/CP_DoModifyAdministrator.pl";
    &Source::DoModifyAdministrator();
  }
  elsif ($action eq "DoLine") {
    require "$UB::SOURCES_PATH/CP_DoLine.pl";
    &Source::DoLine();
  }
  elsif ($action eq "Logout") {
    require "$UB::SOURCES_PATH/CP_DoLogout.pl";
    &Source::DoLogout();
  }

  elsif ($action eq "CreateGroup") {
    require "$UB::SOURCES_PATH/CP_CreateGroup.pl";
    &Source::CreateGroup();
  }
  elsif ($action eq "DoCreateGroup") {
    require "$UB::SOURCES_PATH/CP_DoCreateGroup.pl";
    &Source::DoCreateGroup();
  }
  elsif (($action eq "ListGroups") && ($UB::CGI->param('For'))) {
    require "$UB::SOURCES_PATH/CP_ListGroups.pl";
    &Source::ListGroups();
  }
  elsif (($action eq "ModifyGroup") && ($UB::CGI->param('GID'))) {
    require "$UB::SOURCES_PATH/CP_ModifyGroup.pl";
    &Source::ModifyGroup();
  }
  elsif (($action eq "DoModifyGroup") && ($UB::CGI->param('GID'))) {
    require "$UB::SOURCES_PATH/CP_DoModifyGroup.pl";
    &Source::DoModifyGroup();
  }
  elsif ($action eq "DoRemoveGroups") {
    require "$UB::SOURCES_PATH/CP_DoRemoveGroups.pl";
    &Source::DoRemoveGroups();
  }

  elsif ($action eq "CreateAccount") {
    require "$UB::SOURCES_PATH/CP_CreateAccount.pl";
    &Source::CreateAccount();
  }
  elsif ($action eq "DoCreateAccount") {
    require "$UB::SOURCES_PATH/CP_DoCreateAccount.pl";
    &Source::DoCreateAccount();
  }
  elsif ($action eq "ManageAccounts") {
    require "$UB::SOURCES_PATH/CP_ManageAccounts.pl";
    &Source::ManageAccounts();
  }
  elsif ($action eq "ListAccounts") {
    require "$UB::SOURCES_PATH/CP_ListAccounts.pl";
    &Source::ListAccounts();
  }
  elsif ($action eq "DoDeActivateAccounts") {
    require "$UB::SOURCES_PATH/CP_DoDeActivateAccounts.pl";
    &Source::DoDeActivateAccounts();
  }
  elsif ($action eq "EmailAccounts") {
    require "$UB::SOURCES_PATH/CP_EmailAccounts.pl";
    &Source::EmailAccounts();
  }
  elsif ($action eq "DoEmailAccounts") {
    require "$UB::SOURCES_PATH/CP_DoEmailAccounts.pl";
    &Source::DoEmailAccounts();
  }
  elsif ($action eq "ModifyAccount") {
    require "$UB::SOURCES_PATH/CP_ModifyAccount.pl";
    &Source::ModifyAccount();
  }
  elsif ($action eq "DoModifyAccount") {
    require "$UB::SOURCES_PATH/CP_DoModifyAccount.pl";
    &Source::DoModifyAccount();
  }
  elsif ($action eq "MoveAccounts") {
    require "$UB::SOURCES_PATH/CP_MoveAccounts.pl";
    &Source::MoveAccounts();
  }
  elsif ($action eq "DoMoveAccounts") {
    require "$UB::SOURCES_PATH/CP_DoMoveAccounts.pl";
    &Source::DoMoveAccounts();
  }
  elsif ($action eq "DoRemoveAccounts") {
    require "$UB::SOURCES_PATH/CP_DoRemoveAccounts.pl";
    &Source::DoRemoveAccounts();
  }

  elsif ($action eq "CreateCategory") {
    require "$UB::SOURCES_PATH/CP_CreateCategory.pl";
    &Source::CreateCategory();
  }
  elsif ($action eq "DoCreateCategory") {
    require "$UB::SOURCES_PATH/CP_DoCreateCategory.pl";
    &Source::DoCreateCategory();
  }
  elsif (($action eq "ListCategories") && ($UB::CGI->param('For'))) {
    require "$UB::SOURCES_PATH/CP_ListCategories.pl";
    &Source::ListCategories();
  }
  elsif (($action eq "ModifyCategory") && ($UB::CGI->param('CID'))) {
    require "$UB::SOURCES_PATH/CP_ModifyCategory.pl";
    &Source::ModifyCategory();
  }
  elsif (($action eq "DoModifyCategory") && ($UB::CGI->param('CID'))) {
    require "$UB::SOURCES_PATH/CP_DoModifyCategory.pl";
    &Source::DoModifyCategory();
  }
  elsif ($action eq "DoReorderCategories") {
    require "$UB::SOURCES_PATH/CP_DoReorderCategories.pl";
    &Source::DoReorderCategories();
  }
  elsif ($action eq "DoRemoveCategories") {
    require "$UB::SOURCES_PATH/CP_DoRemoveCategories.pl";
    &Source::DoRemoveCategories();
  }

  elsif ($action eq "CreateBoard") {
    require "$UB::SOURCES_PATH/CP_CreateBoard.pl";
    &Source::CreateBoard();
  }
  elsif ($action eq "DoCreateBoard") {
    require "$UB::SOURCES_PATH/CP_DoCreateBoard.pl";
    &Source::DoCreateBoard();
  }
  elsif (($action eq "ListBoards") && ($UB::CGI->param('For'))) {
    require "$UB::SOURCES_PATH/CP_ListBoards.pl";
    &Source::ListBoards();
  }
  elsif (($action eq "ModifyBoard") && ($UB::CGI->param('BID'))) {
    require "$UB::SOURCES_PATH/CP_ModifyBoard.pl";
    &Source::ModifyBoard();
  }
  elsif (($action eq "DoModifyBoard") && ($UB::CGI->param('BID'))) {
    require "$UB::SOURCES_PATH/CP_DoModifyBoard.pl";
    &Source::DoModifyBoard();
  }
  elsif ($action eq "DoReorderBoards") {
    require "$UB::SOURCES_PATH/CP_DoReorderBoards.pl";
    &Source::DoReorderBoards();
  }
  elsif ($action eq "DoRemoveBoards") {
    require "$UB::SOURCES_PATH/CP_DoRemoveBoards.pl";
    &Source::DoRemoveBoards();
  }

  elsif ($action eq "ManageMessages") {
    require "$UB::SOURCES_PATH/CP_ManageMessages.pl";
    &Source::ManageMessages();
  }
  elsif ($action eq "ListMessages") {
    require "$UB::SOURCES_PATH/CP_ListMessages.pl";
    &Source::ListMessages();
  }
  elsif ($action eq "DoReadOnlyMessages") {
    require "$UB::SOURCES_PATH/CP_DoReadOnlyMessages.pl";
    &Source::DoReadOnlyMessages();
  }
  elsif ($action eq "ModifyMessage") {
    require "$UB::SOURCES_PATH/CP_ModifyMessage.pl";
    &Source::ModifyMessage();
  }
  elsif ($action eq "DoModifyMessage") {
    require "$UB::SOURCES_PATH/CP_DoModifyMessage.pl";
    &Source::DoModifyMessage();
  }
  elsif ($action eq "MoveMessages") {
    require "$UB::SOURCES_PATH/CP_MoveMessages.pl";
    &Source::MoveMessages();
  }
  elsif ($action eq "DoMoveMessages") {
    require "$UB::SOURCES_PATH/CP_DoMoveMessages.pl";
    &Source::DoMoveMessages();
  }
  elsif ($action eq "DoRemoveMessages") {
    require "$UB::SOURCES_PATH/CP_DoRemoveMessages.pl";
    &Source::DoRemoveMessages();
  }

  elsif ($action eq "Backup") {
    require "$UB::SOURCES_PATH/CP_Backup.pl";
    &Source::Backup();
  }
  elsif ($action eq "DoBackup") {
    require "$UB::SOURCES_PATH/CP_DoBackup.pl";
    &Source::DoBackup();
  }
  elsif ($action eq "Restore") {
    require "$UB::SOURCES_PATH/CP_Restore.pl";
    &Source::Restore();
  }
  elsif ($action eq "DoRestore") {
    require "$UB::SOURCES_PATH/CP_DoRestore.pl";
    &Source::DoRestore();
  }
  elsif ($action eq "Rebuild") {
    require "$UB::SOURCES_PATH/CP_Rebuild.pl";
    &Source::Rebuild();
  }
  elsif ($action eq "DoRebuild") {
    require "$UB::SOURCES_PATH/CP_DoRebuild.pl";
    &Source::DoRebuild();
  }
  elsif ($action eq "Compile") {
    if ($UB::MEMBER{'LEVEL'} < 100) {
      require "$UB::SOURCES_PATH/CP_Login.pl";
      &Source::Login();
      exit;
    }

    require "$UB::SOURCES_PATH/CP_Compile.pl";
    &Source::Compile();
  }
  elsif ($action eq "DoCompile") {
    if ($UB::MEMBER{'LEVEL'} < 100) {
      require "$UB::SOURCES_PATH/CP_Login.pl";
      &Source::Login();
      exit;
    }

    require "$UB::SOURCES_PATH/CP_DoCompile.pl";
    &Source::DoCompile();
  }

  elsif ($action eq "HitsReport") {
    require "$UB::SOURCES_PATH/CP_HitsReport.pl";
    &Source::HitsReport();
  }
  elsif ($action eq "AccessReport") {
    require "$UB::SOURCES_PATH/CP_AccessReport.pl";
    &Source::AccessReport();
  }
  elsif ($action eq "AgentsReport") {
    require "$UB::SOURCES_PATH/CP_AgentsReport.pl";
    &Source::AgentsReport();
  }
}

###############################################################################
# CGIError function
# - function of display error message with form and environment information during internal error.
sub CGIError {
  my ($message, $path) = @_;

  my ($key, $space);

  # checking the html header is it printed
  unless ($UB::CONTENT_TYPE == 1) {
    print "Content-type: text/html\n\n";
    $UB::CONTENT_TYPE = 1;
  }

  print "<html>\n";
  print "<head>\n";
  print "<title>CGI Script Error</title>\n";
  print "</head>\n";
  print "<body bgcolor=\"Silver\" text=\"Black\" link=\"Blue\" vlink=\"Purple\" alink=\"Red\" leftmargin=\"8\" topmargin=\"8\" marginwidth=\"8\" marginheight=\"8\">\n";
  print "<font face=\"Verdana\">\n";
  print "<font size=\"4\"><b>CGI Script Error</b></font><p>\n";
  print "<font size=\"2\">\n";

  # printing error message
  $message =~ s/\n/<br>\n/g;
  $message and print $message;
  print "<p>";

  # printing general infomation
  print "<font size=\"4\"><b>General Infomation</b></font><p>\n";
  print "<table cellpadding=\"0\" cellspacing=\"0\">\n";

  if ($path) {
    $path =~ s/\\/\//g;
    print "<tr>\n";
    print "<td width=\"200\" valign=\"TOP\"><font face=\"Verdana\" size=\"2\"><b>Error Path</b></font></td>\n";
    print "<td><font face=\"Verdana\" size=\"2\">$path</font></td>\n";
    print "</tr>\n";
  }
  if ($0) {
    $0 =~ s/\\/\//g;
    print "<tr>\n";
    print "<td width=\"200\" valign=\"TOP\"><font face=\"Verdana\" size=\"2\"><b>Script Path</b></font></td>\n";
    print "<td><font face=\"Verdana\" size=\"2\">$0</font></td>\n";
    print "</tr>\n";
  }
  if ($]) {
    print "<tr>\n";
    print "<td width=\"200\" valign=\"TOP\"><font face=\"Verdana\" size=\"2\"><b>Perl Version</b></font></td>\n";
    print "<td><font face=\"Verdana\" size=\"2\">$]</font></td>\n";
    print "</tr>\n";
  }
  if ($CGI::VERSION) {
    print "<tr>\n";
    print "<td width=\"200\" valign=\"TOP\"><font face=\"Verdana\" size=\"2\"><b>CGI.pm Version</b></font></td>\n";
    print "<td><font face=\"Verdana\" size=\"2\">$CGI::VERSION</font></td>\n";
    print "</tr>\n";
  }
  print "</table><p>\n";

  if (defined %UB::QUERY) {
    # printing form variables
    print "<font size=\"4\"><b>Form Variables</b></font><p>\n";
    print "<table cellpadding=\"0\" cellspacing=\"0\">\n";
    foreach my $KEY (sort keys %UB::QUERY) {
      print "<tr>\n";
      print "<td width=\"200\" valign=\"TOP\"><font face=\"Verdana\" size=\"2\"><b>$KEY</b></font></td>\n";
      print "<td><font face=\"Verdana\" size=\"2\">".$UB::QUERY{$KEY}."</font></td>\n";
      print "</tr>\n";
    }
    print "</table><p>\n";
  }

  # printing environment variables
  print "<font size=\"4\"><b>Environment Variables</b></font><p>\n";
  print "<table cellpadding=\"0\" cellspacing=\"0\">\n";
  foreach $key (sort keys %ENV) {
    print "<tr>\n";
    print "<td width=\"200\" valign=\"TOP\"><font face=\"Verdana\" size=\"2\"><b>$key</b></font></td>\n";
    print "<td><font face=\"Verdana\" size=\"2\">".$ENV{$key}."</font></td>\n";
    print "</tr>\n";
  }
  print "</table><p>\n";
  print "</font>\n";
  print "</font>\n";
  print "</body>\n";
  print "</html>\n";

  exit;
}

package Standard;

###############################################################################
# Authenticate function
# - function of checking the username and password from member
sub Authenticate {
  my ($Profile);
  my ($Username, $Password);

  $UB::MEMBER{'LOGIN'} = 0;

  if (
    ( ( $UB::CGI->param('Username') ) && ( $UB::CGI->param('Username') ne "" ) ) &&
    ( ( $UB::CGI->param('Password') ) && ( $UB::CGI->param('Password') ne "" ) )
  ) {
    $Username = $UB::CGI->param('Username');
    $Password = $UB::CGI->param('Password');

    $Password = &Standard::CryptPassword( $Username, $Password );
  } elsif ( ($UB::COOKIES{'UBCPUsername'}) && ($UB::COOKIES{'UBCPPassword'})) {
    if (
      ( $UB::COOKIES{'UBCPUsername'} ne "" ) &&
      ( $UB::COOKIES{'UBCPPassword'} ne "" )
    ) {
      $Username = $UB::COOKIES{'UBCPUsername'};
      $Password = $UB::COOKIES{'UBCPPassword'};
    }
  } else {
    return;
  }

  $Username = lc ($Username);

  return unless (
    ( $Username ) and ( $Password )
  );

  unless (
    ($Profile) = &DB::BinarySelect (
      TABLE   => "Accounts",
      COLUMNS => [
        "USERNAME"        ,
        "PASSWORD"        ,
        "STATUS"          ,
        "LEVEL"           ,
        "GROUP"           ,
        "LEVEL_TEXT"      ,
        "GROUP_TEXT"      ,
        "NICKNAME"        ,
        "EMAIL"           ,
        "TOTAL_POSTS"     ,
        "REGISTERED_DATE" ,
        "REGISTERED_TIME" ,
        "LAST_POST_DATE"  ,
        "LAST_POST_TIME"  ,
        "LAST_LOGIN_DATE" ,
        "LAST_LOGIN_TIME" ,
        "HOST"            ,
        "IP"              ,
        "APROVED"
      ],
      KEY   => $Username
    )
  ) {
    $UB::COOKIES{'UBCPUsername'} = "";
    $UB::COOKIES{'UBCPPassword'} = "";

    require "$UB::SOURCES_PATH/CP_Login.pl";
    &Source::Login("Invalid username and/or password.");
    exit;
  }

  if ($Profile->{'LEVEL'} < 70) {
    $UB::COOKIES{'UBCPUsername'} = "";
    $UB::COOKIES{'UBCPPassword'} = "";

    require "$UB::SOURCES_PATH/CP_Login.pl";
    &Source::Login("You are not the administrator of this ultraboard.");
    exit;
  }

  my $New = 0;
  if (
    ($Username eq $Profile->{'USERNAME'}) &&
    ($Password eq $Profile->{'PASSWORD'})
  ) {
    my $IP    = $ENV{'REMOTE_ADDR'};
    my $Epoch = time;

    %UB::MEMBER = ( %{ $Profile } );

    $UB::MEMBER{'LOGIN'} = 1;

    $UB::COOKIES{'UBCPUsername'} = $Username;
    $UB::COOKIES{'UBCPPassword'} = $Password;
  } else {
    $UB::COOKIES{'UBCPUsername'} = "";
    $UB::COOKIES{'UBCPPassword'} = "";

    require "$UB::SOURCES_PATH/CP_Login.pl";
    &Source::Login("Invalid username and/or password.");
    exit;
  }


  return 1;
}
