package Remove;

use strict;
use File::Copy;

&Standard::Require("Thread.lib.pl", "$UB::LIBRARIES_PATH/Thread.lib.pl");

sub Post {
  my %in = (
    BOARD  => "",
    POST   => "",
    DETAIL =>  0,
    @_
  );

  my $T;
  my $Topics = 0;
  my $Posts  = 0;

  print "<!--\n" if $in{'DETAIL'} != 1;

  if ( ($in{'BOARD'} eq "") || ($in{'POST'} eq "") ) {
    print "! Missing input value.\n";
    return -1;
  }

  print "- Begin removing post id# $in{'POST'} from board id# $in{'BOARD'}.\n";

  $T = time;
  print "- Getting post information... ";
  (
    my ($Post) = &DB::BinarySelect (
      TABLE   => "B".$in{'BOARD'}         ,
      COLUMNS => ["ID", "ROOT", "REPLIES", "ATTACHMENT"],
      KEY     => $in{'POST'}
    )
  ) || (print "Error: $DB::Error\n" and return -5);
  print "Done. (", (time - $T), "s)\n";

  if ($Post->{'ROOT'} ne "0") {
    print "- Post id# $in{'POST'} is a reply in topic id# $Post->{'ROOT'}.\n";

    $T = time;
    print "- Getting topic id# $Post->{'ROOT'} information... ";
    (
      my ($Topic) = &DB::BinarySelect (
        TABLE   => "B".$in{'BOARD'},
        COLUMNS => ["ID", "REPLIES", "NUMBER_REPLIES", "ATTACHMENT"],
        KEY     => $Post->{'ROOT'}
      )
    ) || (print "Error: $DB::Error\n" and return -5);
    print "Done. (", (time - $T), "s)\n";

    my @Replies = split (/,/, $Topic->{'REPLIES'});
    shift @Replies;

    $T = time;
    print "- Removing post id# $in{'POST'} from topic id# $Topic->{'ID'}... ";
    for (my $index = 0; $index <= $#Replies; $index++) {
      if ($Replies[$index] eq $in{'POST'}) {
        splice (@Replies, $index, 1);
        last;
      }
    }

    $Topic->{'NUMBER_REPLIES'}--;

    (
      &DB::Update (
        TABLE  => "B".$in{'BOARD'},
        VALUES => {
          REPLIES        => ",".join (",", @Replies),
          NUMBER_REPLIES => $Topic->{'NUMBER_REPLIES'},
        },
        KEY    => $Topic->{'ID'}
      )
    ) || (print "Error: $DB::Error\n" and return -5);
    print "Done. (", (time - $T), "s)\n";

    $Posts++;
  } else {
    print "- Post id# $in{'POST'} is a topic.\n";

    my @Replies = split (/,/, $Post->{'REPLIES'});
    shift @Replies;

    if (scalar (@Replies) > 0) {
      $T = time;
      print "- Removing replies from topic id# $Post->{'ID'}.\n";
      for (my $index = 0; $index <= $#Replies; $index++) {
        my ($Reply) = &DB::BinarySelect (
          TABLE   => "B".$in{'BOARD'}    ,
          COLUMNS => ["ID", "ATTACHMENT"],
          KEY     => $Replies[$index]
        );

        if ($Reply->{'ATTACHMENT'} ne "") {
          my $T = time;
          print "- - Removing the reply id# ".$Replies[$index]." attachment in board id# $in{'BOARD'}... ";
          unlink ($SYSTEM::UPLOAD_PATH."/Post-".$in{'BOARD'}."-".$Replies[$index]."-".$Reply->{'ATTACHMENT'}) || print "Error: $! ";
          print "Done. (", (time - $T), "s)\n";
        }

        my $T = time;
        print "- - Removing the reply id# ".$Replies[$index]." body source in board id# $in{'BOARD'}... ";
        unlink ($SYSTEM::DB_PATH."/B".$in{'BOARD'}."/".$Replies[$index].".post") || print "Error: $! ";
        print "Done. (", (time - $T), "s)\n";

        $T = time;
        print "- - Removing the reply id# ".$Replies[$index]." body parsed source in board id# $in{'BOARD'}... ";
        unlink ($SYSTEM::DB_PATH."/B".$in{'BOARD'}."/".$Replies[$index].".src") || print "Error: $! ";
        print "Done. (", (time - $T), "s)\n";

        $Posts++;
      }

      (
        &DB::Delete (
          TABLE  => "B".$in{'BOARD'},
          KEYS   => [@Replies]
        )
      ) || (print "Error: $DB::Error\n" and return -5);
      print "- Done. (", (time - $T), "s)\n";
    }

    $Topics++;
    $Posts++;
  }

  if ($Post->{'ATTACHMENT'} ne "") {
    my $T = time;
    print "- - Removing the reply id# $in{'POST'} attachment in board id# $in{'BOARD'}... ";
    unlink ($SYSTEM::UPLOAD_PATH."/Post-".$in{'BOARD'}."-".$in{'POST'}."-".$Post->{'ATTACHMENT'}) || print "Error: $! ";
    print "Done. (", (time - $T), "s)\n";
  }

  my $T = time;
  print "- Removing the post id# $in{'POST'} html source in board id# $in{'BOARD'}... ";
  my $Counter = 0;
  while (unlink ($SYSTEM::DB_PATH."/B".$in{'BOARD'}."/".$in{'POST'}."-".++$Counter.".html")) {}
  print "Done. (", (time - $T), "s)\n";

  $T = time;
  print "- Removing the post id# $in{'POST'} print source in board id# $in{'BOARD'}... ";
  unlink ($SYSTEM::DB_PATH."/B".$in{'BOARD'}."/".$in{'POST'}.".print");
  print "Done. (", (time - $T), "s)\n";

  $T = time;
  print "- Removing the post id# $in{'POST'} body source in board id# $in{'BOARD'}... ";
  unlink ($SYSTEM::DB_PATH."/B".$in{'BOARD'}."/".$in{'POST'}.".post") || print "Error: $! ";
  print "Done. (", (time - $T), "s)\n";

  $T = time;
  print "- Removing the post id# $in{'POST'} body parsed source in board id# $in{'BOARD'}... ";
  unlink ($SYSTEM::DB_PATH."/B".$in{'BOARD'}."/".$in{'POST'}.".src") || print "Error: $! ";
  print "Done. (", (time - $T), "s)\n";

  $T = time;
  print "- Removing post id# $in{'POST'} information from board id# $in{'BOARD'}... ";
  (
    &DB::Delete (
      TABLE  => "B".$in{'BOARD'},
      KEYS   => [$in{'POST'}]
    )
  ) || (print "Error: $DB::Error\n" and return -5);
  print "Done. (", (time - $T), "s)\n";

  if ($Topics == 0) {
    $T = time;
    print "- Rebuilding topic id# $Post->{'ROOT'} static html source... ";
    &Thread::Build (
      BOARD     => $in{'BOARD'}   ,
      THREAD    => $Post->{'ROOT'},
    );
    print "Done. (", (time - $T), "s)\n";
  }

  $T = time;
  print "- Getting board id# $in{'BOARD'} information... ";
  (
    my ($Board) = &DB::BinarySelect (
      TABLE   => "Boards",
      COLUMNS => ["ID", "CATEGORY"],
      KEY     => $in{'BOARD'}
    )
  ) || (print "Error: $DB::Error\n" and return -5);
  print "Done. (", (time - $T), "s)\n";

  $T = time;
  print "- Updating category id# $Board->{'CATEGORY'} information... ";
  (
    &DB::Update (
      TABLE   => "Categories",
      VALUES  => {
        TOTAL_POSTS         => "\${TOTAL_POSTS}  - $Posts",
        TOTAL_TOPICS        => "\${TOTAL_TOPICS} - $Topics"
      },
      KEY     => $Board->{'CATEGORY'}
    )
  ) || (print "Error: $DB::Error\n" and return -5);
  print "Done. (", (time - $T), "s)\n";

  $T = time;
  print "- Updating board id# $Board->{'ID'} information... ";
  (
    &DB::Update (
      TABLE   => "Boards",
      VALUES  => {
        TOTAL_POSTS         => "\${TOTAL_POSTS}  - $Posts",
        TOTAL_TOPICS        => "\${TOTAL_TOPICS} - $Topics"
      },
      KEY     => $Board->{'ID'}
    )
  ) || (print "Error: $DB::Error\n" and return -5);
  print "Done. (", (time - $T), "s)\n";

  print "-->\n"  if $in{'DETAIL'} != 1;

  return 1;
}