###############################################################################
# Restore.lib.pl                                                              #
###############################################################################
# UltraBoard 2000 by UltraScripts.com, Inc.                                   #
# Scripts written by Jacky Wing Wa Yung, WebMaster@UltraScripts.com           #
# Available from http://www.UltraBoard.org/                                   #
# =========================================================================== #
# COPYRIGHT NOTICE :                                                          #
#                                                                             #
# Copyright (c) 1999-2000 UltraScripts.com, Inc. All Rights Reserved.         #
#                                                                             #
# This program is being distributed as shareware. It may be used and modified #
# free for personal, government or non-profit. Any commercial use must be     #
# registered. However, modified versions cannot be sold or distributed. You   #
# cannot alter the copyright and "powered by" notices throughout the scripts. #
# These notices must be clearly visible to the end users.                     #
#                                                                             #
# WARRANTY DISCLAIMER:                                                        #
#                                                                             #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT #
# ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR       #
# FITNESS FOR A PARTICULAR PURPOSE.                                           #
###############################################################################
package Restore;

use strict;
use Fcntl;
use File::Copy;
use Benchmark;

sub Restore {
  my %in = (
    FROM   => "",
    TO     => "",
    TABLE  => "",
    NUMBER => "",
    CONFIG => [],
    @_
  );

  my $BackupDir  = $in{'FROM'};
  my $RestoreDir = $in{'TO'};

  print "- - ".$in{'NUMBER'}.")Copying the '$in{'TABLE'}' backup directory...\n";

    print "- - - Creating a restore directory for the '$in{'TABLE'}' backup data... ";
    mkdir ("$RestoreDir/$in{'TABLE'}", $SYSTEM::PERMISSION) || (print "Error: $!\n" and return);
    chmod (0777, "$RestoreDir/$in{'TABLE'}");
    print "Done.\n";

    print "- - - Opening the '$in{'TABLE'}' backup directory...\n";
    opendir (DIR, "$BackupDir/$in{'TABLE'}") || (print "Error: $!\n" and return -5);
      while ( defined (my $FILE = readdir (DIR)) ) {
        next if ($FILE =~ /^\.\.?$/);
        next if ($FILE =~ /\.export?$/);
        unless (-d $FILE) {
          print "- - - - Reading the file '$FILE'... ";

          print "copying... ";
          copy ("$BackupDir/$in{'TABLE'}/$FILE", "$RestoreDir/$in{'TABLE'}/$FILE") || print "error: $!... ";
          print "Done.\n";
        }
      }
    closedir (DIR);
    print "- - - Done.\n";

    print "- - - Import the '$in{'TABLE'}' database... ";
    if (open (EXPORT, "$BackupDir/$in{'TABLE'}/$in{'TABLE'}.export")) {
        my @EXPORT = <EXPORT>;
      close (EXPORT);
      chomp (@EXPORT);
      &ImportDatabase(
        FROM   => \@EXPORT,
        TO     => $RestoreDir,
        TABLE  => $in{'TABLE'}
      );
      print "Done.\n";
    } else {
      print "Error: $!\n";
    }

    foreach my $Config ( @{ $in{'CONFIG'} } ) {
      print "- - - Copying the backup config file of '$in{'TABLE'}' database... ";
      copy ("$BackupDir/$Config", "$RestoreDir/$Config") || print "Error: $!\n";
      print "Done.\n";
    }

  print "- - Done.\n\n";

  return 1;
}

###############################################################################
# All function
sub All {
  my %in = (
    TIME   => "",
    DETAIL =>  1,
    @_
  );

  my $T0   = new Benchmark;
  my $TIME = $in{'TIME'};

  print "<!--\n" if $in{'DETAIL'} != 1;

  print "Begin restoring the ultraboard 2000.\n\n";

  print "- Creating a directory for backup '$SYSTEM::BACKUP/UB2K-$TIME'...\n";
  my $BackupDir  = "$SYSTEM::BACKUP/UB2K-$TIME";

  print "- - - Creating a database directory at '$SYSTEM::DB_PATH'... ";
  mkdir ($SYSTEM::DB_PATH, $SYSTEM::PERMISSION)             || print "Error: $!... ";
  chmod (0777, $SYSTEM::DB_PATH);
  print "Done.\n";

  print "- - - Creating a logs directory at '$SYSTEM::LOGS_PATH'... ";
  mkdir ($SYSTEM::LOGS_PATH, $SYSTEM::PERMISSION)           || print "Error: $!... ";
  chmod (0777, $SYSTEM::LOGS_PATH);
  print "Done.\n";

  print "- - - Creating a temp directory at '$SYSTEM::TEMP_PATH'... ";
  mkdir ($SYSTEM::TEMP_PATH, $SYSTEM::PERMISSION)           || print "Error: $!... ";
  chmod (0777, $SYSTEM::TEMP_PATH);
  print "Done.\n";

  print "- Done.\n\n";

  print "- Backing up the member groups data...\n\n";

    my $Counter = 0;

    ###########################################################################
    # Copying the member groups
    &Restore (
      FROM   => $BackupDir,
      TO     => $SYSTEM::DB_PATH,
      TABLE  => "Groups",
      NUMBER => ++$Counter,
      CONFIG => ["Groups.cfg"]
    );

    ###########################################################################
    # Copying the member accounts
    &Restore (
      FROM   => $BackupDir,
      TO     => $SYSTEM::DB_PATH,
      TABLE  => "Accounts",
      NUMBER => ++$Counter,
      CONFIG => ["Accounts.cfg", "accounts.username", "accounts.nickname", "accounts.email"]
    );

    ###########################################################################
    # Copying the member profiles
    &Restore (
      FROM   => $BackupDir,
      TO     => $SYSTEM::DB_PATH,
      TABLE  => "Profiles",
      NUMBER => ++$Counter,
      CONFIG => ["Profiles.cfg"]
    );

    ###########################################################################
    # Copying the member sessions
    &Restore (
      FROM   => $BackupDir,
      TO     => $SYSTEM::DB_PATH,
      TABLE  => "Sessions",
      NUMBER => ++$Counter,
      CONFIG => ["Sessions.cfg"]
    );

    ###########################################################################
    # Copying the password
    &Restore (
      FROM   => $BackupDir,
      TO     => $SYSTEM::DB_PATH,
      TABLE  => "Password",
      NUMBER => ++$Counter,
      CONFIG => ["Password.cfg"]
    );

    ###########################################################################
    # Copying the categories
    &Restore (
      FROM   => $BackupDir,
      TO     => $SYSTEM::DB_PATH,
      TABLE  => "Categories",
      NUMBER => ++$Counter,
      CONFIG => ["Categories.cfg"]
    );

    ###########################################################################
    # Copying the boards
    &Restore (
      FROM   => $BackupDir,
      TO     => $SYSTEM::DB_PATH,
      TABLE  => "Boards",
      NUMBER => ++$Counter,
      CONFIG => ["Boards.cfg"]
    );

    ###########################################################################
    # Copying the board
    print "- - Getting the boards information... ";
    (
      my ($Boards) = &DB::Query (
        TABLE   => "Boards",
        COLUMNS => ["ID"],
      )
    ) || (print "Error: $DB::Error $!\n" and return -5);
    print "Done.\n";

    for (my $index = 0; $index <= $#{ $Boards }; $index++) {
      my $BID = $Boards->[$index]{'ID'};

      &Restore (
        FROM   => $BackupDir,
        TO     => $SYSTEM::DB_PATH,
        TABLE  => "B$BID",
        NUMBER => ++$Counter,
        CONFIG => ["B$BID.cfg"]
      );
    }

  print "- Done.\n\n";

  my $T1 = new Benchmark;

  print "Done. Used".timestr( timediff($T1, $T0) )."\n";

  print "-->\n"  if $in{'DETAIL'} != 1;

  return 1;
}

sub ImportDatabase {
  my %in = (
    FROM   => [],
    TO     => "",
    TABLE  => "",
    @_
  );

  my @ExportData = @{ $in{'FROM'} };
  my %DB = ();

  for (my $index = 1; $index <= $#ExportData; $index++) {
    my ($KEY, $VALUE) = split (/\|\~\|/, $ExportData[$index]);
    $DB{$KEY} = $VALUE;
  }

  tie (my %DBM, $AnyDBM_File::ISA[0], $in{'TO'}."/$in{'TABLE'}/$in{'TABLE'}.db", O_RDWR|O_CREAT, 0777) || (print "Error: $!\n" and return);
    %DBM = ();
    foreach my $KEY (keys %DB) {
      $DBM{$KEY} = $DB{$KEY};
    }
  untie (%DBM);

  return 1;
}

###############################################################################
1; # end of Restore.lib.pl file
###############################################################################