###############################################################################
# Standard.lib.pl                                                             #
###############################################################################
# UltraBoard 2000 by UltraScripts.com, Inc.                                   #
# Scripts written by Jacky Wing Wa Yung, WebMaster@UltraScripts.com           #
# Available from http://www.UltraBoard.org/                                   #
# =========================================================================== #
# COPYRIGHT NOTICE :                                                          #
#                                                                             #
# Copyright (c) 1999-2000 UltraScripts.com, Inc. All Rights Reserved.         #
#                                                                             #
# This program is being distributed as shareware. It may be used and modified #
# free for personal, government or non-profit. Any commercial use must be     #
# registered. However, modified versions cannot be sold or distributed. You   #
# cannot alter the copyright and "powered by" notices throughout the scripts. #
# These notices must be clearly visible to the end users.                     #
#                                                                             #
# WARRANTY DISCLAIMER:                                                        #
#                                                                             #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT #
# ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR       #
# FITNESS FOR A PARTICULAR PURPOSE.                                           #
###############################################################################

package Standard;

use strict;

###############################################################################
# PrintHTMLHeader function
sub PrintHTMLHeader {
  unless ($UB::CONTENT_TYPE == 1) {
    print "Pragma: no-cache\n" if ($_[0] == 1);
    my $Cookies = $UB::CGI->cookie (
      -name    => 'UB2000'     ,
      -value   => \%UB::COOKIES,
      -expires => '+45d'
    );
    my @Cookies = ($Cookies, @UB::TIDS);
    print $UB::CGI->header(-cookie => \@Cookies);
    $UB::CONTENT_TYPE = 1;
  }
}

###############################################################################
# GetBoardsList function
# - function of getting categories and boards list
sub GetBoardsList {
  my ($Categories, $Boards, $BoardsIndex, $Select, $CID, $BID, $BoardsOnly, %Index);

  ($Categories, $Boards, $BoardsIndex, $Select, $CID, $BID, $BoardsOnly) = @_;

  ($Categories) = &DB::Query (
    TABLE   => "Categories"   ,
    COLUMNS => ["ID", "TITLE", "POSITION"],
    SORT    => "POSITION"     ,
    BY      => "A-Z"
  ) unless $Categories ne "";

  CATEGORY: for (my $c = 0; $c <= $#{ $Categories }; $c++) {
    my $Category = $Categories->[$c];

    if ( ( ($UB::MEMBER{'LOGIN'} == 1) && ($UB::MEMBER{'LEVEL'} ne "70") && ($UB::MEMBER{'LEVEL'} ne "100") ) || ($UB::MEMBER{'LOGIN'} == 0)) {
      next CATEGORY unless &Standard::GetCategoryPermission ($Category->{'ID'}, $UB::MEMBER{'LEVEL'}, $UB::MEMBER{'GROUP'});
    }

    @{ $Index{$Category->{'ID'}} } = ();
  }

  ($Boards, $BoardsIndex) = &DB::Query (
    TABLE   => "Boards"       ,
    COLUMNS => ["ID", "TITLE", "STATUS", "CATEGORY", "POSITION"],
    SORT    => "POSITION"     ,
    BY      => "A-Z"
  ) unless (($Boards ne "") && ($BoardsIndex ne ""));

  BOARD: for (my $b = 0; $b <= $#{ $Boards }; $b++) {
    my $Board = $Boards->[$b];
    if ( ( ($UB::MEMBER{'LOGIN'} == 1) && ($UB::MEMBER{'LEVEL'} ne "70") && ($UB::MEMBER{'LEVEL'} ne "100") ) || ($UB::MEMBER{'LOGIN'} == 0)) {
      if ($Board->{'STATUS'} == 50) {
        next BOARD unless &Standard::GetBoardPermission ($Board->{'ID'}, "READ", $UB::MEMBER{'LEVEL'}, $UB::MEMBER{'GROUP'});
      } elsif ($Board->{'STATUS'} == 30) {
        next BOARD;
      } elsif ($Board->{'STATUS'} != 70) {
        next BOARD;
      }
    }

    if ( exists $Index { $Board->{'CATEGORY'} } ) {
      push (@{ $Index { $Board->{'CATEGORY'} } }, $Board->{'ID'});
    }
  }

  my ($SelectOptions);
  if ($Select == 1) {
    $SelectOptions .= "<option>Go to...</option>";
    CATEGORY: for (my $c = 0; $c <= $#{ $Categories }; $c++) {
      my $Category = $Categories->[$c];

      if ( exists $Index { $Category->{'ID'} } ) {
        unless ($BoardsOnly == 1) {
          $SelectOptions .= "<option></option>";

          $SelectOptions .= "<option value=\"action=Category&CID=".$Category->{'ID'}."\"";
          $SelectOptions .= " selected" if $Category->{'ID'} eq $CID;
          $SelectOptions .= ">".$Category->{'TITLE'}."</option>";

          $SelectOptions .= "<option>===========================</option>";
        }

        BOARD: for (my $b = 0; $b <= $#{ $Index { $Category->{'ID'} } }; $b++ ) {
          my $Board = $Boards->[ $BoardsIndex->{ $Index { $Category->{'ID'} }->[$b] } ];

          $SelectOptions .= "<option value=\"action=Headlines&BID=".$Board->{'ID'}."\"";
          $SelectOptions .= " selected" if $Board->{'ID'} eq $BID;
          $SelectOptions .= ">".$Board->{'TITLE'}."</option>";
        }
      }
    }
  }

  return ($Select == 1) ? (wantarray ? ($SelectOptions, \%Index) : $SelectOptions) : \%Index;
}

###############################################################################
# GetSession function
# - function of getting session
sub GetSession {
  my ($ID) = @_;
  my ($Session);

  unless (
    ($Session) = &DB::BinarySelect (
      TABLE => "Sessions",
      KEY   => $ID
    )
  ) {
    return;
  }

  return if ($Session->{'IP'} ne $ENV{'REMOTE_ADDR'});

  return $Session;
}

###############################################################################
# WordFilter function
# - function of filter out all words that set
sub WordFilter {
  my (@texts) = @_;

  my (@Censors) = split (/\|/, $GENERAL::WORD_FILTER);

  foreach (@Censors) {
    my ($fWord, $fOption, $fBecome) = split (/\:/, $_);
    my ($expression);

    if ($fBecome eq "") {
      $fBecome = "#" x length($fWord);
    }
    $fWord =~ s/\\/\\\\/g;
    $fWord =~ s/\|/\\\|/g;
    $fWord =~ s/\@/\\\@/g;
    $fWord =~ s/\$/\\\$/g;
    $fWord =~ s/\%/\\\%/g;
    $fWord =~ s/\~/\\\~/g;

    $fBecome =~ s/\\/\\\\/g;
    $fBecome =~ s/\|/\\\|/g;
    $fBecome =~ s/\@/\\\@/g;
    $fBecome =~ s/\$/\\\$/g;
    $fBecome =~ s/\%/\\\%/g;
    $fBecome =~ s/\~/\\\~/g;

    if ($fOption eq "b") {
      $expression = eval "sub { s/\b$fWord\b/$fBecome/ig }";
    } else {
      $expression = eval "sub { s/$fWord/$fBecome/ig }";
    }

    foreach (@texts) {
      &{$expression};
    }
  }

  return wantarray ? @texts : $texts[0];
}

###############################################################################
# UsernameFilter function
# - function of filter out all username that set
sub UsernameFilter {
  my ($Username) = @_;

  my @Usernames = split (/\|/, $GENERAL::USERNAME_FILTER);

  foreach (@Usernames) {
    my ($fUsername, $fOption) = split (/\:/, $_);

    if ($fOption eq "b") {
      if ($Username =~ /^$fUsername$/i) {
        return 1;
      }
    } else {
      if ($Username =~ /$fUsername/i) {
        return 1;
      }
    }
  }

  return;
}

###############################################################################
# NicknameFilter function
# - function of filter out all nickname that set
sub NicknameFilter {
  my ($Nickname) = @_;

  my @Nicknames = split (/\|/, $GENERAL::NICKNAME_FILTER);

  foreach (@Nicknames) {
    my ($fNickname, $fOption) = split (/\:/, $_);

    if ($fOption eq "b") {
      if ($Nickname =~ /^$fNickname$/i) {
        return 1;
      }
    } else {
      if ($Nickname =~ /$fNickname/i) {
        return 1;
      }
    }
  }

  return;
}

###############################################################################
# GetCategoryPermission function
# - function of getting permission of the category
sub GetCategoryPermission {
  my ($ID, $Level, $Group) = @_;

  $Level = "00" unless $Level;

  &Standard::FileOpen(*DATA, "rw", "$SYSTEM::DB_PATH/Categories/$ID-LEVEL-$Level.dat");
    my @Groups = <DATA>;
  close (DATA);
  chomp (@Groups);

  return 1 if ($Groups[0] eq "*");

  if ("$Level" eq "00") {
    return;
  }

  foreach (@Groups) {
    if ($_ eq $Group) {
      return 1;
    }
  }

  return;
}

###############################################################################
# GetBoardPermission function
# - function of getting permission of the board
sub GetBoardPermission {
  my ($ID, $Type, $Level, $Group) = @_;

  $Level = "00" unless $Level;

  &Standard::FileOpen(*DATA, "rw", "$SYSTEM::DB_PATH/B".$ID."/$Type-LEVEL-$Level.dat");
    my @Groups = <DATA>;
  close (DATA);
  chomp (@Groups);

  return 1 if ($Groups[0] eq "*");

  if ("$Level" eq "00") {
    return;
  }

  foreach (@Groups) {
    if ($_ eq $Group) {
      return 1;
    }
  }

  return;
}

###############################################################################
# EmailFilter function
# - function of filter out all email that set
sub EmailFilter {
  my ($Email) = @_;

  my @Emails = split (/\|/, $GENERAL::EMAIL_FILTER);

  my ($eUsername, $eDomain) = split (/\@/, $Email);

  foreach (@Emails) {
    my ($fEmail, $fOption);
    ($fEmail, $fOption) = split (/\:/, $_, 2);
    $fOption = "" unless defined $fOption;

    if ($fOption eq "db") {
      if ($eDomain =~ /^$fEmail$/i) {
        return 1;
      }
    } elsif ($fOption eq "ub") {
      if ($eUsername =~ /^$fEmail$/i) {
        return 1;
      }
    } elsif ($fOption eq "d") {
      if ($eDomain =~ /$fEmail/i) {
        return 1;
      }
    } elsif ($fOption eq "u") {
      if ($eUsername =~ /$fEmail/i) {
        return 1;
      }
    } else {
      if ($Email =~ /^$fEmail$/i) {
        return 1;
      }
    }
  }

  return;
}

###############################################################################
# EncodeHTML function
sub EncodeHTML {
  my ($data) = @_;

  if (ref ($data) eq "ARRAY") {
    for (my $i = 0; $i <= $#{$data}; $i++) {
      $data->[$i] =~ s/\&/\&amp\;/g;
      $data->[$i] =~ s/\</\&lt\;/g;
      $data->[$i] =~ s/\>/\&gt\;/g;
      #$data->[$i]  =~ s/\"/\&quot\;/g;
      #$data->[$i] =~ s/\'/\&\#39\;/g;
      $data->[$i] =~ s/\r//g;
    }
  } elsif (ref ($data) eq "HASH") {
    foreach my $key (keys %{$data}) {
      $data->{$key} =~ s/\&/\&amp\;/g;
      $data->{$key} =~ s/\</\&lt\;/g;
      $data->{$key} =~ s/\>/\&gt\;/g;
      #$data->{$key}  =~ s/\"/\&quot\;/g;
      #$data->{$key} =~ s/\'/\&\#39\;/g;
      $data->{$key} =~ s/\r//g;
    }
  } else {
    $data =~ s/\&/\&amp\;/g;
    $data =~ s/\</\&lt\;/g;
    $data =~ s/\>/\&gt\;/g;
    #$data  =~ s/\"/\&quot\;/g;
    #$data =~ s/\'/\&\#39\;/g;
    $data =~ s/\r//g;
  }

  return $data;
}

###############################################################################
# DecodeHTML function
sub DecodeHTML {
  my ($data) = @_;

  if (ref ($data) eq "ARRAY") {
    for (my $i = 0; $i <= $#{$data}; $i++) {
      $data->[$i] =~ s/\&\#39\;/\'/g;
      $data->[$i] =~ s/\&quot\;/\"/g;
      $data->[$i] =~ s/\&gt\;/\>/g;
      $data->[$i] =~ s/\&lt\;/\</g;
      $data->[$i] =~ s/\&amp\;/\&/g;
    }
  } elsif (ref ($data) eq "HASH") {
    foreach my $key (keys %{$data}) {
      $data->{$key} =~ s/\&\#39\;/\'/g;
      $data->{$key} =~ s/\&quot\;/\"/g;
      $data->{$key} =~ s/\&gt\;/\>/g;
      $data->{$key} =~ s/\&lt\;/\</g;
      $data->{$key} =~ s/\&amp\;/\&/g;
    }
  } else {
    $data =~ s/\&\#39\;/\'/g;
    $data =~ s/\&quot\;/\"/g;
    $data =~ s/\&gt\;/\>/g;
    $data =~ s/\&lt\;/\</g;
    $data =~ s/\&amp\;/\&/g;
  }

  return $data;
}

###############################################################################
# UnReturn function
sub UnReturn {
  my ($data) = @_;

  if (ref ($data) eq "ARRAY") {
    for (my $i = 0; $i <= $#{$data}; $i++) {
      $data->[$i] =~ s/\n/\\n/g;
    }
  } elsif (ref ($data) eq "HASH") {
    foreach my $key (keys %{$data}) {
      $data->{$key} =~ s/\n/\\n/g;
    }
  } else {
    $data =~ s/\n/\\n/g;
  }

  return $data;
}

###############################################################################
# RandomPassword function
# - function of creating a random password.
sub RandomPassword {
  my @Chars = (
    "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m",
    "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z",

    "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M",
    "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z",

    "1", "2", "3", "4", "5", "6", "7", "8", "9", "0"
  );

  my $Password;

  srand (time);

  for (my $i = 0; $i < 6; $i++) {
    $Password .= $Chars[ int ( rand ( $#Chars + 1 ) ) ];
  }

  return $Password;
}

###############################################################################
# CryptPassword function
# - function of encrypt password.
sub CryptPassword {
  my ($UserName, $Password) = @_;

  return unless ($UserName or $Password);

  &Standard::Require("Crypt.pm", "$UB::LIBRARIES_PATH/Crypt.pm");

  return Crypt::crypt ($Password, lc ( substr ( $UserName, 0, 2 ) ) );
}

###############################################################################
# Require function
# - function of requiring library.
sub Require {
  my ($Library, $Path) = @_;

  return if ($UB::LIBRARY{$Library});

  eval {
    require "$Path";
  };

  # error during load library
  &UB::CGIError("Can't load the \"$Library\" library. $@", $Path) if ($@);

  $UB::LIBRARY{$Library} = 1;

  return 1;
}

###############################################################################
# InsertToList function
# - function of inserting data into list
sub InsertToList {
  my ($Path, $Data) = @_;

  &Standard::lock("file.lock") if ($SYSTEM::FLOCK == 1);

  (-e $Path) ?
  ( &FileOpen(*LIST, "a", $Path) ) :
  ( &FileOpen(*LIST, "w", $Path) ) ;

    print LIST $Data."\n";
  close (LIST);

  &Standard::unLock("file.lock") if ($SYSTEM::FLOCK == 1);
}

###############################################################################
# UpdateFromList function
# - function of updating data from list
sub UpdateFromList {
  my ($Path, $Old, $New) = @_;

  my @List = &GetList($Path);

  &Standard::lock("file.lock") if ($SYSTEM::FLOCK == 1);

  &FileOpen(*LIST, "w", $Path);

  for (my $i = 0; $i <= $#List; $i++) {
    if ($List[$i] eq $Old) {
      $List[$i] = $New;
    }
    print LIST $List[$i]."\n" if ($List[$i] ne "");
  }

  close (LIST);

  &Standard::unLock("file.lock") if ($SYSTEM::FLOCK == 1);
}

###############################################################################
# GetList function
# - function of getting data from list
sub GetList {
  my ($Path) = @_;

  my @Data;
  unless (-e $Path) {
    &FileOpen(*LIST, "w", $Path);
  } else {
    &FileOpen(*LIST, "r", $Path);
      @Data = <LIST>;
  }
  close (LIST);

  chomp (@Data);

  return @Data;
}

###############################################################################
# FileOpen function
# - function of open a file for better error handling.
sub FileOpen {
  my ($FileHandle, $AccessMode, $FileName, $skip) = @_;
  $FileName   =~ s/\|//g;
  $FileName   =~ s/\>//g;
  $FileName   =~ s/\<//g;

  $FileName =~ /^(.+)\/[^\/]+$/;
  my $Path = $1;

  my $Mode;
  if ($AccessMode eq "a") {
    if (-e $FileName) {
      $Mode = ">>";
    } else {
      $Mode = ">";
    }
  } elsif ($AccessMode eq "w") {
    $Mode = ">";
  } elsif ($AccessMode eq "r") {
    $Mode = "";
  } elsif ($AccessMode eq "rw") {
    unless (-e $FileName) {
      open ($FileHandle, ">".$FileName);
      close ($FileHandle);
    }
    $Mode = "";
  }

  chmod (0766, $FileName);

  unless (open ($FileHandle, $Mode.$FileName)) {
    if ($AccessMode eq "a") {
      &UB::CGIError("Can't open file for appending. $!", $FileName) unless $skip;
      return;
    } elsif ($AccessMode eq "w") {
      &UB::CGIError("Can't open file for writing. $!"  , $FileName) unless $skip;
      return;
    } elsif ($AccessMode eq "r") {
      &UB::CGIError("Can't open file for reading. $!"  , $FileName) unless $skip;
      return;
    }
  }

  chmod (0766, $FileName);

  if ($SYSTEM::FLOCK == 1) {
    if ($AccessMode ne "r") {
      flock ($FileHandle, 2);
    } else {
      flock ($FileHandle, 1);
    }
  }

  return 1;
}

###############################################################################
# lock function
# - function of locking files without using build-in function 'flock()'.
sub lock {
  my ($file)  = @_;
  my $EndTime = 30 + time;

  while ((-e $file) && (time < $EndTime)) {
    sleep(1);
  }

  chmod (0777, $SYSTEM::TEMP_PATH);

  open(LOCK, ">$SYSTEM::TEMP_PATH/$file") || &UB::CGIError("Can't open a file for locking. $!", "$SYSTEM::TEMP_PATH/$file");
    chmod (0777, "$SYSTEM::TEMP_PATH/$file");
}

###############################################################################
# unLock function
# - function of unlocking files without using build-in function 'flock()'.
sub unLock {
  my ($file) = @_;

  close(LOCK);

  unlink("$SYSTEM::TEMP_PATH/$file");
}

###############################################################################
# ConvertEpochToDate function
# - function of convert epoch second to date, and time.
sub ConvertEpochToDate {
  my ($epoch) = @_;

  $epoch = time unless $epoch;

  my @ShortMonths = ("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec");
  my ($mDay, $mon, $year, $isdst);

  if (
    ($SYSTEM::LOCALTIME_OFFSET ne "" ) ||
    ($SYSTEM::LOCALTIME_OFFSET ne "0")
  ) {
    (undef, undef, undef, $mDay, $mon, $year, undef, undef, $isdst) = localtime ($epoch + ($SYSTEM::LOCALTIME_OFFSET * 3600));
  } else {
    (undef, undef, undef, $mDay, $mon, $year, undef, undef, $isdst) = localtime ($epoch);
  }

  my $Date;

  $year += 1900;

  if ($SYSTEM::DATE_FORMAT eq "US") {
    $Date = sprintf ("%02d-%02d-%04d", $mon + 1, $mDay, $year);
  } elsif ($SYSTEM::DATE_FORMAT eq "USE") {
    $Date = sprintf ("%s %02d, %04d", $ShortMonths[$mon], $mDay, $year);
  } elsif ($SYSTEM::DATE_FORMAT eq "EU") {
    $Date = sprintf ("%02d-%02d-%04d", $mDay, $mon + 1, $year);
  } elsif ($SYSTEM::DATE_FORMAT eq "EUE") {
    $Date = sprintf ("%02d %s, %04d", $mDay, $ShortMonths[$mon], $year);
  } else {
    $Date = sprintf ("%02d-%02d-%04d", $mon + 1, $mDay, $year);
  }

  return $Date;
}

###############################################################################
# ConvertEpochToTime function
# - function of convert epoch second to date, and time.
sub ConvertEpochToTime {
  my ($epoch) = @_;

  $epoch = time unless $epoch;

  my ($sec, $min, $hour, $isdst);

  if (
    ($SYSTEM::LOCALTIME_OFFSET ne "" ) ||
    ($SYSTEM::LOCALTIME_OFFSET ne "0")
  ) {
    ($sec, $min, $hour, undef, undef, undef, undef, undef, $isdst) = localtime ($epoch + ($SYSTEM::LOCALTIME_OFFSET * 3600));
  } else {
    ($sec, $min, $hour, undef, undef, undef, undef, undef, $isdst) = localtime ($epoch);
  }

  my $Time;

  if ($SYSTEM::TIME_FORMAT eq "12") {
    if ($hour >= 12) {
      $hour -= 12 if ($hour > 12);
      $Time = sprintf ("%02d:%02d p.m.", $hour, $min);
    } else {
      $Time = sprintf ("%02d:%02d a.m.", $hour, $min);
    }
  } else {
    $Time = sprintf ("%02d:%02d", $hour, $min);
  }

  return $Time;
}

###############################################################################
# ceil function
sub ceil {
  my ($x, $y) = @_;

  if ($x % $y == 0) {
    return $x / $y;
  } else {
    return int ($x / $y + 1);
  }
}

###############################################################################
# Chmod function
sub Chmod {
  my ($D, $Permission) = @_;
  my (@chmods);

  return unless $D;

  if ($Permission eq "r") {
    @chmods = (0766, 0776, 0755, 0775, 0777);
  } elsif ($Permission eq "w") {
    @chmods = (0766, 0776, 0755, 0775, 0777);
  } elsif ($Permission eq "x") {
    @chmods = (0755, 0775, 0777);
  }

  if (-d $D) {
    for (my $i = 0; $i <= $#chmods; $i++) {
      if (opendir(TEST, $D)) {
        closedir(TEST);
        last;
      } else {
        if ($! =~ /Permission denied/i) {
          chmod ($chmods[$i], $D);
        } else {
          last;
        }
      }
    }
    unless (opendir(TEST, $D)) {
      closedir(TEST);
      return;
    }
  } else {
    for (my $i = 0; $i <= $#chmods; $i++) {
      if (open(TEST, $D)) {
        close(TEST);
        last;
      } else {
        if ($! =~ /Permission denied/i) {
          my ($Path, $File) = ($D =~ /^(.+)\/([^\/]+)$/);
          DIR: for (my $d = 0; $d <= $#chmods; $d++) {
            chmod ($chmods[$d], $Path);

            FILE: for (my $f = 0; $f <= $#chmods; $f++) {
              unless (-r $File) {
                chmod ($chmods[$f], $File);
              } else {
                last FILE;
              }
            }

            last DIR if (-r $File);
          }
        } else {
          last;
        }
      }
    }

    unless (open(TEST, $D)) {
      close(TEST);
      return;
    }
  }


  if ($Permission eq "r") {
    for (my $i = 0; $i <= $#chmods; $i++) {
      unless (-r $D) {
        chmod ($chmods[$i], $D);
      } else {
        last;
      }
    }

    return unless (-r $D);
  } elsif ($Permission eq "w") {
    for (my $i = 0; $i <= $#chmods; $i++) {
      unless (-w $D) {
        chmod ($chmods[$i], $D);
      } else {
        last;
      }
    }

    return unless (-w $D);
  } elsif ($Permission eq "x") {
    for (my $i = 0; $i <= $#chmods; $i++) {
      unless (-x $D) {
        chmod ($chmods[$i], $D);
      } else {
        last;
      }
    }

    return unless (-x $D);
  }

  return 1;
}

###############################################################################
# HostIPFilter function
sub HostIPFilter {
  if ($SYSTEM::IP_FILTER) {
    my @IPS = split (/\|/, $SYSTEM::IP_FILTER);

    foreach my $IP (@IPS) {
      if ($ENV{'REMOTE_ADDR'} =~ /^$IP/) {
        return;
      }
    }
  }

  if (($SYSTEM::HOST_FILTER) && ($ENV{'REMOTE_HOST'})) {
    my @HOSTS = split (/\|/, $SYSTEM::HOST_FILTER);

    foreach my $HOST (@HOSTS) {
      if ($ENV{'REMOTE_HOST'} =~ /$HOST$/) {
        return;
      }
    }
  }

  return 1;
}

###############################################################################
# trim function
sub trim {
  for (my $i = 0; $i <= $#_; $i++) {
    $_[$i] =~ s/\s+$//g;
    $_[$i] =~ s/^\s+//g;
  }
  return wantarray ? @_ : $_[0];
}

###############################################################################
# trimTrailing function
sub trimTrailing {
  for (my $i = 0; $i <= $#_; $i++) {
    $_[$i] =~ s/\s+$//g;
  }
  return wantarray ? @_ : $_[0];
}

###############################################################################
# HTML Functions
package HTML;

###############################################################################
# Checkbox function
# - function of printing checkbox
sub Checkbox {
  my %args = (
    NAME    => "",
    VALUE   => "",
    CHECKED => "",
    EXTRA   => "",
    @_
  );

  my $HTML;

  $args{'NAME'} = qq~ name="$args{'NAME'}"~ if $args{'NAME'} ne "";

  if (($args{'VALUE'} eq $args{'CHECKED'}) && ($args{'CHECKED'} ne "")) {
    $args{'CHECKED'}  =qq~ checked~;
  } else {
    $args{'CHECKED'}  ="";
  }

  $args{'VALUE'}  =qq~ value="$args{'VALUE'}"~  if $args{'VALUE'} ne "";

  $HTML=qq~<input type="Checkbox"$args{'NAME'}$args{'VALUE'}$args{'CHECKED'} class="Checkbox" $args{'EXTRA'}>~;

  return $HTML;
}

###############################################################################
# Radio function
# - function of printing radio
sub Radio {
  my %args = (
    NAME      => "",
    VALUE     => "",
    CHECKED   => "",
    EXTRA     => "",
    @_
  );

  my $HTML;

  $args{'NAME'} = qq~ name="$args{'NAME'}"~ if $args{'NAME'} ne "";

  if (($args{'VALUE'} eq $args{'CHECKED'}) && ($args{'CHECKED'} ne "")) {
    $args{'CHECKED'}  =qq~ checked~;
  } else {
    $args{'CHECKED'}  ="";
  }

  $args{'VALUE'}  =qq~ value="$args{'VALUE'}"~  if $args{'VALUE'} ne "";

  $HTML=qq~<input type="Radio"$args{'NAME'}$args{'VALUE'}$args{'CHECKED'} class="Radio" $args{'EXTRA'}>~;

  return $HTML;
}

###############################################################################
# Select function
# - function of printing select
sub Select {
  my %args = (
    NAME      => "",
    VALUE     => [],
    SIZE      => "",
    SELECTED  => [],
    MULTIPLE  => "",
    EXTRA     => "",
    @_
  );

  $args{'SELECTED'} = join ("|", @{ $args{'SELECTED'} });
  $args{'SELECTED'} = "\|".$args{'SELECTED'}."\|";

  my $HTML;

  $args{'NAME'}     = qq~ name="$args{'NAME'}"~ if $args{'NAME'}     ne "";
  $args{'SIZE'}     = qq~ size="$args{'SIZE'}"~ if $args{'SIZE'}     ne "";
  $args{'MULTIPLE'} = qq~ multiple~             if $args{'MULTIPLE'} ne "";

  $HTML = qq~<select$args{'NAME'}$args{'SIZE'} class="Select"$args{'MULTIPLE'} $args{'EXTRA'}>~;
  for (my $i = 0; $i <= $#{ $args{'VALUE'} }; $i++) {
    $HTML .= qq~<option value="$args{'VALUE'}->[$i]{'KEY'}"~;
    if ($args{'SELECTED'} =~ /\Q|$args{'VALUE'}->[$i]{'KEY'}|\E/) {
      $HTML .= qq~ selected~;
    }
    $HTML .= qq~>$args{'VALUE'}->[$i]{'VALUE'}</option>~;
  }
  $HTML .= qq~</select>~;

  return $HTML;
}

###############################################################################
1; # end of Standard.lib.pl file
###############################################################################