###############################################################################
# UB_DoLostPassword.pl                                                        #
###############################################################################
# UltraBoard 2000 by UltraScripts.com, Inc.                                   #
# Scripts written by Jacky Wing Wa Yung, WebMaster@UltraScripts.com           #
# Available from http://www.UltraBoard.org/                                   #
# =========================================================================== #
# COPYRIGHT NOTICE :                                                          #
#                                                                             #
# Copyright (c) 1999-2000 UltraScripts.com, Inc. All Rights Reserved.         #
#                                                                             #
# This program is being distributed as shareware. It may be used and modified #
# free for personal, government or non-profit. Any commercial use must be     #
# registered. However, modified versions cannot be sold or distributed. You   #
# cannot alter the copyright and "powered by" notices throughout the scripts. #
# These notices must be clearly visible to the end users.                     #
#                                                                             #
# WARRANTY DISCLAIMER:                                                        #
#                                                                             #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT #
# ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR       #
# FITNESS FOR A PARTICULAR PURPOSE.                                           #
###############################################################################

package Source;

use strict;

sub DoLostPassword {

  my $SendMail = 0;
  if ($SYSTEM::SENDMAIL == 1) {
    if ( ($SYSTEM::MAIL_SYSTEM eq "SMTP") && ($SYSTEM::SMTP_PATH ne "") ) {
      $SendMail = &Standard::Require("SMTP.lib.pl", "$UB::LIBRARIES_PATH/SMTP.lib.pl");
    } elsif ( ($SYSTEM::MAIL_SYSTEM eq "SENDMAIL") && ($SYSTEM::SENDMAIL_PATH ne "") ) {
      $SendMail = &Standard::Require("Sendmail.lib.pl", "$UB::LIBRARIES_PATH/Sendmail.lib.pl");
    }
  }

  &HTML::Error (CODE => "FUNCTION_DISABLED") unless ($SendMail);

# =========================================================================== #
# Verify Variables

  # checking missing fields
  if ($UB::QUERY{'FORM_USERNAME'} eq "") { &Error("MISS_USERNAME") };
  if ($UB::QUERY{'FORM_EMAIL'}    eq "") { &Error("MISS_EMAIL"   ) };
  if ($UB::QUERY{'FORM_QUESTION'} eq "") { &Error("MISS_QUESTION") };
  if ($UB::QUERY{'FORM_ANSWER'}   eq "") { &Error("MISS_ANSWER"  ) };

  (
    my ($Account) = &DB::BinarySelect (
      TABLE   => "Accounts",
      COLUMNS => ["USERNAME", "EMAIL", "APROVED", "STATUS"],
      KEY     => lc ($UB::QUERY{'FORM_USERNAME'})
    )
  ) || &Error("ACCOUNT_NOT_FOUND");

  unless ($Account->{'APROVED'} == 1) {
    &HTML::Error (CODE => "ACCOUNT_NOT_APPROVED");
  }

  unless ($Account->{'STATUS'} == 50) {
    &HTML::Error (CODE => "ACCOUNT_INACTIVATE");
  }

  &Error("INVALID_EMAIL")    if ($Account->{'EMAIL'} ne $UB::QUERY{'FORM_EMAIL'});
  &Error("INVALID_QUESTION") if (length ($UB::QUERY{'FORM_QUESTION'}) < 10);
  &Error("INVALID_ANSWER")   if (length ($UB::QUERY{'FORM_ANSWER'})   <  2);

# =========================================================================== #
# Assign Variables

  my %FORM;

  $FORM{'USERNAME'}          = lc ($UB::QUERY{'FORM_USERNAME'});
  $FORM{'EMAIL'}             = lc ($UB::QUERY{'FORM_EMAIL'});
  $FORM{'QUESTION'}          = $UB::QUERY{'FORM_QUESTION'};
  $FORM{'ANSWER'}            = lc ($UB::QUERY{'FORM_ANSWER'});
  $FORM{'CLEAN_ACTIVE_CODE'} = &Standard::RandomPassword();
  $FORM{'ACTIVE_CODE'}       = &Standard::CryptPassword($FORM{'USERNAME'}, $FORM{'CLEAN_ACTIVE_CODE'});

  $FORM{'SECOND'}            = time;
  $FORM{'DATE'}              = &Standard::ConvertEpochToDate($FORM{'SECOND'});
  $FORM{'TIME'}              = &Standard::ConvertEpochToTime($FORM{'SECOND'});

# =========================================================================== #
# Add Member to Database

  (
    my $ID = &DB::Insert (
      TABLE  => "Password",
      VALUES => \%FORM
    )
  ) || &HTML::Error(MESSAGE => $DB::Error);

  my %PASSWORD = (
    "PASSWORD.ID"          => $ID              ,
    "PASSWORD.USERNAME"    => $FORM{'USERNAME'},
    "PASSWORD.EMAIL"       => $FORM{'EMAIL'}   ,
    "PASSWORD.QUESTION"    => $FORM{'QUESTION'},
    "PASSWORD.ANSWER"      => $FORM{'ANSWER'}  ,
    "PASSWORD.ACTIVE_CODE" => $FORM{'CLEAN_ACTIVE_CODE'}
  );

  if (($SYSTEM::SENDMAIL == 1) && ($SendMail == 1)) {
    &Standard::Require("UB_Active_Code_Mail.src", "$UB::INTERFACES_PATH/$SYSTEM::DEFAULT_THEME/UB_Active_Code_Mail.src");
    &Mail::Send (
      SUBJECT => $Active_Code_Mail::DYNAMIC{'LANGUAGE.SUBJECT'},
      MESSAGE => &Active_Code_Mail::Parse(\%PASSWORD),
#     HEADER  => $GENERAL::EMAIL_FORMAT,
      TO      => $FORM{'EMAIL'}
    );
  }

  &Success(\%PASSWORD);
}

###############################################################################
# Error function
# - function of printing error message of sending active code
sub Error {
  my ($code) = @_;

  require "$UB::SOURCES_PATH/UB_LostPassword.pl";
  &Source::LostPassword($code);

  exit;
}

###############################################################################
# Success function
# - function of printing success message of sending active code
sub Success {
  my ($PASSWORD) = @_;

  &Standard::Require("UB_Lost_Password_Success.src", "$UB::INTERFACES_PATH/$SYSTEM::DEFAULT_THEME/UB_Lost_Password_Success.src");

  &Standard::PrintHTMLHeader();
  print &Lost_Password_Success::Parse($PASSWORD);
  exit;
}

###############################################################################
1; # end of UB_DoLostPassword.pl file
###############################################################################