#!#%PERL.PATH%#

###############################################################################
# UltraBoard.cgi                                                              #
###############################################################################
# UltraBoard 2000 by UltraScripts.com, Inc.                                   #
# Scripts written by Jacky Wing Wa Yung, WebMaster@UltraScripts.com           #
# Available from http://www.UltraBoard.org/                                   #
# =========================================================================== #
# COPYRIGHT NOTICE :                                                          #
#                                                                             #
# Copyright (c) 1999-2000 UltraScripts.com, Inc. All Rights Reserved.         #
#                                                                             #
# This program is being distributed as shareware. It may be used and modified #
# free for personal, government or non-profit. Any commercial use must be     #
# registered. However, modified versions cannot be sold or distributed. You   #
# cannot alter the copyright and "powered by" notices throughout the scripts. #
# These notices must be clearly visible to the end users.                     #
#                                                                             #
# WARRANTY DISCLAIMER:                                                        #
#                                                                             #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT #
# ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR       #
# FITNESS FOR A PARTICULAR PURPOSE.                                           #
###############################################################################

package UB;

use CGI qw (:cgi);

use strict;

use Benchmark;

my $T0 = new Benchmark;

BEGIN {
  # if you having problem on the script, set $UB::PATH to the full path to your
  # ultraboard directory.
  $UB::PATH    = "#%UB.PATH%#";

  $SYSTEM::EXT = "#%PERL.EXT%#" || $ENV{'SCRIPT_NAME'} || $ENV{'SCRIPT_FILENAME'} || $0;
  $SYSTEM::EXT =~s/^.+\.(\w+)$/$1/g;

  ($0 =~ m/(.*)\/[^\/]+/) && unshift (@INC, "$1");
  ($0 =~ m/(.*)\\[^\\]+/) && unshift (@INC, "$1");

  eval {
    unless (-e $UB::PATH."/UltraBoard.$SYSTEM::EXT") {
      foreach my $P ( $ENV{'SCRIPT_FILENAME'}, $ENV{'PATH_TRANSLATED'}, $ENV{'DOCUMENT_ROOT'}.$ENV{'SCRIPT_NAME'}, $0 ) {
        $P =~s/\\/\//g;
        $P =~s/(.*)\/[^\/]+/$1/g;
        if (-e $P."/UltraBoard.$SYSTEM::EXT") {
          $UB::PATH = $P;
          last;
        }
      }
    }
  };

  unshift (@INC, "$UB::PATH");
}

# required using at least perl v5.004.
eval {
  require 5.004;
} || &UB::CGIError("Can't load the perl v5.004 or higher version. $@", "");

###############################################################################
# Variables

$| = 1;

$UB::VERSION         = "2.00";

$UB::SOURCES_PATH    = "$UB::PATH/Private/Sources";
$UB::VARIABLES_PATH  = "$UB::PATH/Private/Variables";
$UB::LIBRARIES_PATH  = "$UB::SOURCES_PATH/Libraries";
$UB::INTERFACES_PATH = "$UB::PATH/Private/Interfaces";

$UB::CONTENT_TYPE    = 0;

%UB::MEMBER          = ();
%UB::COOKIES         = ();
%UB::QUERY           = ();
@UB::TIDS            = ();

push (@INC, $UB::LIBRARIES_PATH);

###############################################################################
# Loading Required Libraries

unless (my $Return = do "$UB::VARIABLES_PATH/System.cfg") {
  &UB::CGIError("Couldn't parse system variables. $@", "$UB::VARIABLES_PATH/System.cfg") if ($@);
  &UB::CGIError("Couldn't do system variables. $_"   , "$UB::VARIABLES_PATH/System.cfg") unless (defined $Return);
  &UB::CGIError("Couldn't run system variables."     , "$UB::VARIABLES_PATH/System.cfg") unless ($Return);
}

unless (my $Return = do "$UB::VARIABLES_PATH/General.cfg") {
  &UB::CGIError("Couldn't parse general variables. $@", "$UB::VARIABLES_PATH/General.cfg") if ($@);
  &UB::CGIError("Couldn't do general variables. $_"   , "$UB::VARIABLES_PATH/General.cfg") unless (defined $Return);
  &UB::CGIError("Couldn't run general variables"      , "$UB::VARIABLES_PATH/General.cfg") unless ($Return);
}

unless (my $Return = do "$SYSTEM::THEME_PATH/$SYSTEM::DEFAULT_THEME/Style/Style.cfg") {
  &UB::CGIError("Couldn't parse style variables. $@", "$SYSTEM::THEME_PATH/$SYSTEM::DEFAULT_THEME/Style/Style.cfg") if ($@);
  &UB::CGIError("Couldn't do style variables. $_"   , "$SYSTEM::THEME_PATH/$SYSTEM::DEFAULT_THEME/Style/Style.cfg") unless (defined $Return);
  &UB::CGIError("Couldn't run style variables"      , "$SYSTEM::THEME_PATH/$SYSTEM::DEFAULT_THEME/Style/Style.cfg") unless ($Return);
}

# loading standard library.
eval { require "$UB::LIBRARIES_PATH/Standard.lib.pl"       } || &UB::CGIError("Can't load standard libraries for ultraboard. $@", "$UB::LIBRARIES_PATH/Standard.lib.pl"      );
# loading database library.
eval { require "$UB::LIBRARIES_PATH/Database.lib.pl"       } || &UB::CGIError("Can't load database libraries for ultraboard. $@", "$UB::LIBRARIES_PATH/Database.lib.pl"      );

if (($SYSTEM::LOGS) || ($SYSTEM::STATS)) {
  # loading database library.
  eval { require "$UB::LIBRARIES_PATH/Track.lib.pl"       } || &UB::CGIError("Can't load track libraries for ultraboard. $@", "$UB::LIBRARIES_PATH/Track.lib.pl"      );
}

my $T1 = new Benchmark;

$UB::CGI = new CGI;

my $T2 = new Benchmark;

# error running main function
eval { &UB::Main } || &UB::CGIError("Can't run your request. $@", "");

my $T3 = new Benchmark;

print "\n<!--\n";

print "Libraries Processing Time: " . timestr( timediff($T1, $T0) ) . "\n";
print "CGI.pm Processing Time   : " . timestr( timediff($T2, $T1) ) . "\n";
print "Request Processing Time  : " . timestr( timediff($T3, $T2) ) . "\n";
print "Script Processing Time   : " . timestr( timediff($T3, $T0) ) . "\n";

print "-->\n";

exit;

###############################################################################
# Main function
# - function of executing all functions
sub Main {

  %UB::QUERY = ();
  foreach my $Keyword ($UB::CGI->param()) {
    $UB::QUERY{ $Keyword } = $UB::CGI->param( $Keyword );
    $UB::QUERY{ $Keyword } =~ s/^\s+//g;
    $UB::QUERY{ $Keyword } =~ s/\s+$//g;
  }

  %UB::COOKIES = $UB::CGI->cookie('UB2000');

  my $action = $UB::QUERY{'action'};

  if (-e "$SYSTEM::LOGS_PATH/clean.log") {
    my $mtime = (stat("$SYSTEM::LOGS_PATH/clean.log"))[9];
    if ($mtime < (time - (60 * 60 * 6))) {
      &Standard::Require("Clean.lib.pl", "$UB::LIBRARIES_PATH/Clean.lib.pl");
      &Clean::Sessions();
      &Clean::Temp();

      if ($SYSTEM::LOGS) {
        &Standard::FileOpen(*LOG, "a", "$SYSTEM::LOGS_PATH/clean.log");
          print LOG time."\n";
        close (LOG);
      }
    }
  } else {
    open (CLEAN, ">$SYSTEM::LOGS_PATH/clean.log");
    close(CLEAN);
  }

  &Standard::Authenticate();

  if (($SYSTEM::LOGS) || ($SYSTEM::STATS)) {
    &Standard::Track();
  }

  unless (&Standard::HostIPFilter()) {
    &HTML::Error (CODE => "BANNED");
  }

  if (-e "$SYSTEM::DB_PATH/offline") {
    &HTML::Error( MESSAGE => $SYSTEM::OFFLINE_MESSAGE );
  }

  unless (-e "$UB::INTERFACES_PATH/$SYSTEM::DEFAULT_THEME") {
    &UB::CGIError("You may forgot to compile $UB::INTERFACES_PATH/$SYSTEM::DEFAULT_THEME theme from your control panel.", "");
  }

  if    (!$action) {
    if ($GENERAL::INDEX != 1) {
      print "Location: $GENERAL::INDEX_URL\n\n";
      exit;
    }
    if ($GENERAL::LOGIN == 1) {
      unless ($UB::MEMBER{'LOGIN'}) {
        print "Location: $SYSTEM::UB_URL/UltraBoard.$SYSTEM::EXT?action=Login\n\n";
        exit;
      }
    }
    require "$UB::SOURCES_PATH/UB_Index.pl";
    &Source::Index;
  }
  elsif ( ($action eq "Category") && ($UB::CGI->param('CID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_Category.pl";
    &Source::Category;
  }

  elsif ( ($action eq "Headlines") && ($UB::CGI->param('BID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_Headlines.pl";
    &Source::Headlines;
  }

  elsif ($action eq "Login") {
    require "$UB::SOURCES_PATH/UB_Login.pl";
    &Source::Login;
  }
  elsif ($action eq "Logout") {
    require "$UB::SOURCES_PATH/UB_Logout.pl";
    &Source::Logout;
  }
  elsif ($action eq "LostPassword") {
    require "$UB::SOURCES_PATH/UB_LostPassword.pl";
    &Source::LostPassword;
  }
  elsif ($action eq "DoLostPassword") {
    require "$UB::SOURCES_PATH/UB_DoLostPassword.pl";
    &Source::DoLostPassword;
  }
  elsif ( ($action eq "ActiveCode") && ($UB::CGI->param('PID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_ActiveCode.pl";
    &Source::ActiveCode;
  }
  elsif ( ($action eq "DoActiveCode") && ($UB::CGI->param('PID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_DoActiveCode.pl";
    &Source::DoActiveCode;
  }

  elsif ($action eq "Register") {
    require "$UB::SOURCES_PATH/UB_Register.pl";
    &Source::Register;
  }
  elsif ($action eq "DoRegister") {
    require "$UB::SOURCES_PATH/UB_DoRegister.pl";
    &Source::DoRegister;
  }
  elsif ($action eq "ModifyProfile") {
    require "$UB::SOURCES_PATH/UB_ModifyProfile.pl";
    &Source::ModifyProfile;
  }
  elsif ($action eq "DoModifyProfile") {
    require "$UB::SOURCES_PATH/UB_DoModifyProfile.pl";
    &Source::DoModifyProfile;
  }
  elsif ( ($action eq "Profile") && ($UB::CGI->param('UID') ne "")) {
    require "$UB::SOURCES_PATH/UB_Profile.pl";
    &Source::Profile;
  }

  elsif ( ($action eq "Email") && ($UB::CGI->param('UID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_Email.pl";
    &Source::Email;
  }
  elsif ( ($action eq "DoEmail") && ($UB::CGI->param('UID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_DoEmail.pl";
    &Source::DoEmail;
  }

  elsif ( ($action eq "Post") && ($UB::CGI->param('BID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_Post.pl";
    &Source::Post;
  }
  elsif ( ($action eq "DoPost") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('PREVIEW') ne "") ) {
    require "$UB::SOURCES_PATH/UB_Preview.pl";
    &Source::Preview;
  }
  elsif ( ($action eq "DoPost") && ($UB::CGI->param('BID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_DoPost.pl";
    &Source::DoPost;
  }

  elsif ( ($action eq "Reply") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('TID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_Reply.pl";
    &Source::Reply;
  }
  elsif ( ($action eq "DoReply") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('TID') ne "") && ($UB::CGI->param('PREVIEW') ne "") ) {
    require "$UB::SOURCES_PATH/UB_Preview.pl";
    &Source::Preview;
  }
  elsif ( ($action eq "DoReply") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('TID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_DoReply.pl";
    &Source::DoReply;
  }

  elsif ( ($action eq "ModifyPost") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('PID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_ModifyPost.pl";
    &Source::ModifyPost;
  }
  elsif ( ($action eq "DoModifyPost") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('PID') ne "") && ($UB::CGI->param('PREVIEW') ne "") ) {
    require "$UB::SOURCES_PATH/UB_Preview.pl";
    &Source::Preview;
  }
  elsif ( ($action eq "DoModifyPost") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('PID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_DoModifyPost.pl";
    &Source::DoModifyPost;
  }

  elsif ( ($action eq "Read") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('TID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_Read.pl";
    &Source::Read;
  }
  elsif ( ($action eq "Print") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('TID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_Read.pl";
    &Source::Read;
  }
  elsif ( ($action eq "Forward") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('TID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_Forward.pl";
    &Source::Forward;
  }
  elsif ( ($action eq "DoForward") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('TID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_DoForward.pl";
    &Source::DoForward;
  }

  elsif ($action eq "Search") {
    require "$UB::SOURCES_PATH/UB_Search.pl";
    &Source::Search;
  }
  elsif ($action eq "DoSearch") {
    require "$UB::SOURCES_PATH/UB_DoSearch.pl";
    &Source::DoSearch;
  }
  elsif ($action eq "Result") {
    require "$UB::SOURCES_PATH/UB_Result.pl";
    &Source::Result;
  }

  elsif ( ($action eq "TopicCommands") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('TID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_TopicCommands.pl";
    &Source::TopicCommands;
  }
  elsif ( ($action eq "ModifyTopic") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('PID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_ModifyTopic.pl";
    &Source::ModifyTopic;
  }
  elsif ( ($action eq "DoModifyTopic") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('PID') ne "") && ($UB::CGI->param('PREVIEW') ne "") ) {
    require "$UB::SOURCES_PATH/UB_Preview.pl";
    &Source::Preview;
  }
  elsif ( ($action eq "DoModifyTopic") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('PID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_DoModifyTopic.pl";
    &Source::DoModifyTopic;
  }
  elsif ( ($action eq "DoCloseTopic") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('TID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_DoCloseTopic.pl";
    &Source::DoCloseTopic;
  }
  elsif ( ($action eq "DoMoveTopic") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('ToBID') ne "") && ($UB::CGI->param('TID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_DoMoveTopic.pl";
    &Source::DoMoveTopic;
  }
  elsif ( ($action eq "DoRemoveTopic") && ($UB::CGI->param('BID') ne "") && ($UB::CGI->param('TID') ne "") ) {
    require "$UB::SOURCES_PATH/UB_DoRemoveTopic.pl";
    &Source::DoRemoveTopic;
  }
  elsif ($action eq "Help") {
    require "$UB::SOURCES_PATH/UB_Help.pl";
    &Source::Help;
  }

  else {
    if ($GENERAL::INDEX != 1) {
      print "Location: $GENERAL::INDEX_URL\n\n";
      exit;
    }
    if ($GENERAL::LOGIN == 1) {
      if ($UB::MEMBER{'LEVEL'}) {
        print "Location: $SYSTEM::UB_URL/UltraBoard.$SYSTEM::EXT?action=Login\n\n";
        exit;
      }
    }
    require "$UB::SOURCES_PATH/UB_Index.pl";
    &Source::Index;
  }
}

###############################################################################
# CGIError function
# - function of display error message with form and environment information during internal error.
sub CGIError {
  my ($message, $path) = @_;

  my ($key, $space);

  # checking the html header is it printed
  unless ($UB::CONTENT_TYPE == 1) {
    print "Content-type: text/html\n\n";
    $UB::CONTENT_TYPE = 1;
  }

  print "<html>\n";
  print "<head>\n";
  print "<title>CGI Script Error</title>\n";
  print "</head>\n";
  print "<body>\n";
  print "<font face=\"Verdana\">\n";
  print "<font size=\"4\"><b>CGI Script Error</b></font><p>\n";
  print "<font size=\"2\">\n";

  # printing error message
  $message =~ s/\n/<br>\n/g;
  $message and print $message;
  print "<p>";

  # printing general infomation
  print "<font size=\"4\"><b>General Infomation</b></font><p>\n";
  print "<table cellpadding=\"0\" cellspacing=\"0\">\n";

  if ($path) {
    $path =~ s/\\/\//g;
    print "<tr>\n";
    print "<td width=\"200\" valign=\"TOP\"><font face=\"Verdana\" size=\"2\"><b>Error Path</b></font></td>\n";
    print "<td><font face=\"Verdana\" size=\"2\">$path</font></td>\n";
    print "</tr>\n";
  }
  if ($0) {
    $0 =~ s/\\/\//g;
    print "<tr>\n";
    print "<td width=\"200\" valign=\"TOP\"><font face=\"Verdana\" size=\"2\"><b>Script Path</b></font></td>\n";
    print "<td><font face=\"Verdana\" size=\"2\">$0</font></td>\n";
    print "</tr>\n";
  }
  if ($]) {
    print "<tr>\n";
    print "<td width=\"200\" valign=\"TOP\"><font face=\"Verdana\" size=\"2\"><b>Perl Version</b></font></td>\n";
    print "<td><font face=\"Verdana\" size=\"2\">$]</font></td>\n";
    print "</tr>\n";
  }
  if ($CGI::VERSION) {
    print "<tr>\n";
    print "<td width=\"200\" valign=\"TOP\"><font face=\"Verdana\" size=\"2\"><b>CGI.pm Version</b></font></td>\n";
    print "<td><font face=\"Verdana\" size=\"2\">$CGI::VERSION</font></td>\n";
    print "</tr>\n";
  }
  print "</table><p>\n";

  if (defined %UB::QUERY) {
    # printing form variables
    print "<font size=\"4\"><b>Form Variables</b></font><p>\n";
    print "<table cellpadding=\"0\" cellspacing=\"0\">\n";
    foreach my $KEY (sort keys %UB::QUERY) {
      print "<tr>\n";
      print "<td width=\"200\" valign=\"TOP\"><font face=\"Verdana\" size=\"2\"><b>$KEY</b></font></td>\n";
      print "<td><font face=\"Verdana\" size=\"2\">".$UB::QUERY{$KEY}."</font></td>\n";
      print "</tr>\n";
    }
    print "</table><p>\n";
  }

  # printing environment variables
  print "<font size=\"4\"><b>Environment Variables</b></font><p>\n";
  print "<table cellpadding=\"0\" cellspacing=\"0\">\n";
  foreach $key (sort keys %ENV) {
    print "<tr>\n";
    print "<td width=\"200\" valign=\"TOP\"><font face=\"Verdana\" size=\"2\"><b>$key</b></font></td>\n";
    print "<td><font face=\"Verdana\" size=\"2\">".$ENV{$key}."</font></td>\n";
    print "</tr>\n";
  }
  print "</table><p>\n";
  print "</font>\n";
  print "</font>\n";
  print "</body>\n";
  print "</html>\n";

  exit;
}

package HTML;

sub Error {
  my %args = (
    CODE    => "",
    MESSAGE => "",
    @_
  );

  &Standard::Require("UB_Error.src", "$UB::INTERFACES_PATH/$SYSTEM::DEFAULT_THEME/UB_Error.src");

  my %FORM = (
    "ERROR.MESSAGE" => $Error::DYNAMIC{"LANGUAGE.".$args{'CODE'}} || $args{'MESSAGE'}
  );

  &Standard::PrintHTMLHeader();
  print &Error::Parse(\%FORM);
  exit;
}

package Standard;

###############################################################################
# Authenticate function
# - function of checking the username and password from member
sub Authenticate {
  my ($Profile);
  my ($Username, $Password);

  $UB::MEMBER{'LOGIN'} = 0;

  if (
    ( ( $UB::CGI->param('Username') ) && ( $UB::CGI->param('Username') ne "" ) ) &&
    ( ( $UB::CGI->param('Password') ) && ( $UB::CGI->param('Password') ne "" ) )
  ) {
    $Username = $UB::CGI->param('Username');
    $Password = $UB::CGI->param('Password');

    $Password = &Standard::CryptPassword( $Username, $Password );
  } elsif (
    ( $UB::CGI->param('SID') ne ""                                   ) &&
    ( my $Session = &Standard::GetSession ( $UB::CGI->param('SID') ) )
  ) {
    $Username = $Session->{'USERNAME'};
    $Password = $Session->{'PASSWORD'};

    $UB::MEMBER{'SESSION'}  = $UB::CGI->param('SID');
  } elsif (
    ( $UB::COOKIES{'SID'} ne ""                                   ) &&
    ( my $Session = &Standard::GetSession ( $UB::COOKIES{'SID'} ) )
  ) {
    $Username = $Session->{'USERNAME'};
    $Password = $Session->{'PASSWORD'};

    $UB::MEMBER{'SESSION'}  = $UB::COOKIES{'SID'};
  } elsif ( ($UB::COOKIES{'Username'}) && ($UB::COOKIES{'Password'})) {
    #} elsif ( ($UB::CGI->cookie('Username')) && ($UB::CGI->cookie('Password'))) {
    #  ( $UB::CGI->cookie('Username') ne "" ) &&
    #  ( $UB::CGI->cookie('Password') ne "" )
    if (
      ( $UB::COOKIES{'Username'} ne "" ) &&
      ( $UB::COOKIES{'Password'} ne "" )
    ) {
      #$Username = $UB::CGI->cookie('Username');
      #$Password = $UB::CGI->cookie('Password');
      $Username = $UB::COOKIES{'Username'};
      $Password = $UB::COOKIES{'Password'};
    }
  } else {
    return;
  }

  $Username = lc ($Username);

  return unless (
    ( $Username ) and ( $Password )
  );

  unless (
    ($Profile) = &DB::BinarySelect (
      TABLE   => "Accounts",
      COLUMNS => [
        "USERNAME"        ,
        "PASSWORD"        ,
        "STATUS"          ,
        "LEVEL"           ,
        "GROUP"           ,
        "LEVEL_TEXT"      ,
        "GROUP_TEXT"      ,
        "NICKNAME"        ,
        "EMAIL"           ,
        "TOTAL_POSTS"     ,
        "REGISTERED_DATE" ,
        "REGISTERED_TIME" ,
        "LAST_POST_DATE"  ,
        "LAST_POST_TIME"  ,
        "LAST_LOGIN_DATE" ,
        "LAST_LOGIN_TIME" ,
        "HOST"            ,
        "IP"              ,
        "APROVED"
      ],
      KEY   => $Username
    )
  ) {
    require "$UB::SOURCES_PATH/UB_Login.pl";
    &Source::Login("INVALID_LOGIN");
    exit;
  }

  unless ($Profile->{'APROVED'} == 1) {
    &HTML::Error (CODE => "ACCOUNT_NOT_APPROVED");
  }

  unless ($Profile->{'STATUS'} == 50) {
    &HTML::Error (CODE => "ACCOUNT_INACTIVATE");
  }

  my $New = 0;
  if (
    ($Username eq $Profile->{'USERNAME'}) &&
    ($Password eq $Profile->{'PASSWORD'})
  ) {
    my $IP    = $ENV{'REMOTE_ADDR'};
    my $Epoch = time;

    %UB::MEMBER = ( %UB::MEMBER,  %{ $Profile } );

    $UB::MEMBER{'LOGIN'} = 1;

    if ($UB::MEMBER{'SESSION'} ne "") {
      (
        &DB::Update (
          TABLE   => "Sessions",
          VALUES  => {
            USERNAME      => $Username,
            PASSWORD      => $Password,
            LOGIN_SECOND  => $Epoch   ,
            IP            => $IP
          },
          KEY     => $UB::MEMBER{'SESSION'}
        )
      ) || &HTML::Error(MESSAGE => $DB::Error);
    } else {
      $New = 1;
      my $Date = &Standard::ConvertEpochToDate($Epoch);
      my $Time = &Standard::ConvertEpochToTime($Epoch);
      my $Host = $ENV{'REMOTE_HOST'};

      (
        $UB::MEMBER{'SESSION'} = &DB::Insert (
          TABLE  => "Sessions",
          VALUES => {
            USERNAME      => $Username,
            PASSWORD      => $Password,
            LOGIN_SECOND  => $Epoch   ,
            IP            => $IP
          }
        )
      ) || &HTML::Error(MESSAGE => $DB::Error);

      (
        &DB::Update (
          TABLE   => "Accounts",
          VALUES  => {
            LAST_LOGIN_SECOND => $Epoch,
            LAST_LOGIN_DATE   => $Date ,
            LAST_LOGIN_TIME   => $Time ,
            HOST              => $Host ,
            IP                => $IP
          },
          KEY     => $Username
        )
      ) || &HTML::Error(MESSAGE => $DB::Error);
    }

    if ($UB::MEMBER{'SESSION'}) {
      $UB::COOKIES{'SID'} = $UB::MEMBER{'SESSION'};
    }

    if (
      ($UB::CGI->param('FORM_REMEMBER_PASSWORD') ==    1) ||
      ($UB::CGI->param('FORM_REMEMBER_PASSWORD') eq "on")
    ) {
      #push (
      #  @UB::COOKIES,
      #  $UB::CGI->cookie(
      #    -name     => "Username",
      #    -value    => $Username,
      #    -expires  => '+1Y'
      #  )
      #);

      #push (
      #  @UB::COOKIES,
      #  $UB::CGI->cookie(
      #    -name     => "Password",
      #    -value    => $Password,
      #    -expires  => '+1Y'
      #  )
      #);

      $UB::COOKIES{'Username'} = $Username;
      $UB::COOKIES{'Password'} = $Password;
    }
  } else {
    require "$UB::SOURCES_PATH/UB_Login.pl";
    &Source::Login("INVALID_LOGIN");
    exit;
  }

  #print "Location: $SYSTEM::UB_URL/UltraBoard.$SYSTEM::EXT?action=".$UB::CGI->param('action')."&BID=".$UB::CGI->param('BID')."&CID=".$UB::CGI->param('CID')."&TID=".$UB::CGI->param('TID')."&SID=".$UB::MEMBER{'SESSION'}."&DayPrune=".$UB::CGI->param('DayPrune')."&P=".$UB::CGI->param('P')."\n\n" if $New == 1;

  return 1;
}

###############################################################################
# ExtraVariables function
# - function of return extra variables
sub ExtraVariables {

  my %EXTRA = (
    "GENERAL.SITE.NAME"               => $GENERAL::SITE_NAME                        ,
    "GENERAL.SITE.URL"                => $GENERAL::SITE_URL                         ,
    "GENERAL.ULTRABOARD.NAME"         => $GENERAL::ULTRABOARD_NAME                  ,
    "GENERAL.ULTRABOARD.URL"          => $SYSTEM::UB_URL."/UltraBoard.".$SYSTEM::EXT,
    "GENERAL.ULTRABOARD.DESCRIPTION"  => $GENERAL::ULTRABOARD_DESCRIPTION           ,
    "GENERAL.IMAGES.URL"              => $SYSTEM::IMAGE_URL."/".$SYSTEM::DEFAULT_THEME,
    "GENERAL.UPLOAD.URL"              => $SYSTEM::UPLOAD_URL                        ,
    "GENERAL.CP.URL"                  => $SYSTEM::CP_URL."/Panel.".$SYSTEM::EXT     ,
    "GENERAL.OTHERS.URL"              => $SYSTEM::UB_URL."/Others.".$SYSTEM::EXT      ,
    "GENERAL.CONTACT_EMAIL"           => $GENERAL::CONTACT_EMAIL                    ,
    "GENERAL.TIMEZONE"                => $SYSTEM::TIMEZONE                          ,
    "GENERAL.SHOW_IP_HOST"            => $GENERAL::SHOW_IP_HOST                     ,

    "GENERAL.THEME"                   => $SYSTEM::DEFAULT_THEME                     ,

    "GENERAL.CATEGORY_ONLY"           => $GENERAL::CATEGORY_ONLY                    ,

    "GENERAL.PRINT"                   => $GENERAL::PRINT                            ,
    "GENERAL.FORWARD"                 => $GENERAL::FORWARD                          ,

    "GENERAL.DISPLAY_AGREEMENT"       => $GENERAL::DISPLAY_AGREEMENT                ,
    "GENERAL.AGREEMENT"               => $GENERAL::AGREEMENT                        ,

    "GENERAL.VERIFY_EMAIL"            => $GENERAL::VERIFY_EMAIL                     ,
    "GENERAL.REVIEW_ACCOUNT"          => $GENERAL::REVIEW_ACCOUNT                   ,

    "GENERAL.NICKNAME_DUPLICATE"      => $GENERAL::NICKNAME_DUPLICATE               ,
    "GENERAL.EMAIL_DUPLICATE"         => $GENERAL::EMAIL_DUPLICATE                  ,

    "GENERAL.NOTIFY_MEMBER_POST"      => $GENERAL::NOTIFY_MEMBER_POST               ,

    "GENERAL.MAX_SUBJECT_LENGTH"      => $GENERAL::MAX_SUBJECT_LENGTH               ,
    "GENERAL.MAX_MESSAGE_LENGTH"      => $GENERAL::MAX_MESSAGE_LENGTH               ,

    "GENERAL.MAX_PHOTO_LENGTH"        => $GENERAL::MAX_PHOTO_LENGTH                 ,
    "GENERAL.MAX_PHOTO_X"             => $GENERAL::MAX_PHOTO_X                      ,
    "GENERAL.MAX_PHOTO_Y"             => $GENERAL::MAX_PHOTO_Y                      ,

    "GENERAL.DATE"                    => \&Standard::ConvertEpochToDate()           ,
    "GENERAL.TIME"                    => \&Standard::ConvertEpochToTime()           ,

    "ACTION"                          => $UB::CGI->param('action') || ""            ,
    "REFER"                           => $UB::CGI->param('refer') || ""             ,

    "MEMBER.LOGIN"                    => $UB::MEMBER{'LOGIN'} || ""                 ,
    "MEMBER.SESSION"                  => $UB::MEMBER{'SESSION'} || ""               ,
    "MEMBER.USERNAME"                 => $UB::MEMBER{'USERNAME'} || ""              ,

    "MEMBER.GROUP"                    => $UB::MEMBER{'GROUP'} || ""                 ,
    "MEMBER.LEVEL"                    => $UB::MEMBER{'LEVEL'} || ""                 ,
    "MEMBER.STATUS"                   => $UB::MEMBER{'STATUS'} || ""                ,

    "MEMBER.NICKNAME"                 => $UB::MEMBER{'NICKNAME'} || ""              ,
    "MEMBER.EMAIL"                    => $UB::MEMBER{'EMAIL'} || ""                 ,

    "MEMBER.TOTAL_POSTS"              => $UB::MEMBER{'TOTAL_POSTS'} || ""           ,

    "MEMBER.REGISTERED_DATE"          => $UB::MEMBER{'REGISTERED_DATE'} || ""       ,
    "MEMBER.REGISTERED_TIME"          => $UB::MEMBER{'REGISTERED_TIME'} || ""       ,

    "MEMBER.LAST_POST_DATE"           => $UB::MEMBER{'LAST_POST_DATE'} || ""        ,
    "MEMBER.LAST_POST_TIME"           => $UB::MEMBER{'LAST_POST_TIME'} || ""        ,

    "MEMBER.LAST_LOGIN_DATE"          => $UB::MEMBER{'LAST_LOGIN_DATE'} || ""       ,
    "MEMBER.LAST_LOGIN_TIME"          => $UB::MEMBER{'LAST_LOGIN_TIME'} || ""       ,

    "MEMBER.HOST"                     => $ENV{'REMOTE_HOST'}                        ,
    "MEMBER.IP"                       => $ENV{'REMOTE_ADDR'}
  );

  foreach my $KEY (keys %UB::STYLE) {
    $EXTRA{ "STYLE.".$KEY } = $UB::STYLE{ $KEY };
  }

  foreach my $KEY (keys %EXTRA) {
    $EXTRA{$KEY} =~ s/\"/\\\"/g;
    $EXTRA{$KEY} =~ s/\$/\\\$/g;
    $EXTRA{$KEY} =~ s/\@/\\\@/g;
    $EXTRA{$KEY} =~ s/\%/\\\%/g;
  }

  return %EXTRA;
}